Class: RDoc::AnyMethod

Inherits:
MethodAttr show all
Includes:
TokenStream
Defined in:
lib/rdoc/code_object/any_method.rb,
lib/rdoc/generator/markup.rb

Overview

AnyMethod is the base class for objects representing methods

Constant Summary collapse

MARSHAL_VERSION =
2

RDoc 4 Added calls_super Added parent name and class Added section title

3

RDoc 4.1 Added is_alias_for

4

Added type_signature_lines (serialized as joined string)

4

Constants included from Text

Text::MARKUP_FORMAT, Text::SPACE_SEPARATED_LETTER_CLASS

Instance Attribute Summary collapse

Attributes inherited from MethodAttr

#aliases, #block_params, #params, #singleton, #type_signature_lines, #visibility

Attributes inherited from CodeObject

#comment, #document_children, #document_self, #done_documenting, #file, #force_documentation, #line, #metadata, #mixin_from, #parent, #received_nodoc, #section, #store

Attributes included from Text

#language

Instance Method Summary collapse

Methods included from TokenStream

#add_token, #add_tokens, #collect_tokens, #pop_token, #source_language, to_html, #token_stream, #tokens_to_s

Methods inherited from MethodAttr

#<=>, #==, #add_line_numbers, #add_location_comment, #aref, #documented?, #find_method_or_attribute, #find_see, #full_name, #html_name, #initialize_copy, #initialize_visibility, #inspect, #markup_code, #name_ord_range, #name_prefix, #parent_name, #path, #pretty_name, #pretty_print, #search_record, #search_snippet, #see, #to_s, #type

Methods inherited from CodeObject

#display?, #documented?, #file_name, #full_name=, #ignore, #ignored?, #initialize_visibility, #options, #parent_name, #record_location, #start_doc, #stop_doc, #suppress, #suppressed?

Methods included from Generator::Markup

#aref_to, #as_href, #canonical_url, #cvs_url, #description, #formatter

Methods included from Text

decode_legacy_label, expand_tabs, #flush_left, #markup, #normalize_comment, #parse, #snippet, #strip_hashes, #strip_newlines, #strip_stars, to_anchor, #wrap

Constructor Details

#initialize(name, singleton: false) ⇒ AnyMethod

Creates a new AnyMethod with name name



45
46
47
48
49
50
51
52
53
# File 'lib/rdoc/code_object/any_method.rb', line 45

def initialize(name, singleton: false)
  super(name, singleton: singleton)

  @c_function = nil
  @dont_rename_initialize = false
  @token_stream = nil
  @calls_super = false
  @superclass_method = nil
end

Instance Attribute Details

#c_function ⇒ Object

The C function that implements this method (if it was defined in a C file)



30
31
32
# File 'lib/rdoc/code_object/any_method.rb', line 30

def c_function
  @c_function
end

#calls_super ⇒ Object

If true this method uses super to call a superclass version



38
39
40
# File 'lib/rdoc/code_object/any_method.rb', line 38

def calls_super
  @calls_super
end

#dont_rename_initialize ⇒ Object

Don't rename #initialize to ::new



25
26
27
# File 'lib/rdoc/code_object/any_method.rb', line 25

def dont_rename_initialize
  @dont_rename_initialize
end

#section_title ⇒ Object

The section title of the method (if defined in a C file via :category:)



33
34
35
# File 'lib/rdoc/code_object/any_method.rb', line 33

def section_title
  @section_title
end

Instance Method Details

#add_alias(an_alias, context = nil) ⇒ Object

Adds an_alias as an alias for this method in context.



58
59
60
61
62
63
64
65
66
67
68
69
70
# File 'lib/rdoc/code_object/any_method.rb', line 58

def add_alias(an_alias, context = nil)
  method = self.class.new an_alias.new_name, singleton: singleton

  method.record_location an_alias.file
  method.params = self.params
  method.visibility = self.visibility
  method.comment = an_alias.comment
  method.is_alias_for = self
  method.type_signature_lines = self.type_signature_lines
  @aliases << method
  context.add_method method if context
  method
end

#aref_prefix ⇒ Object

Prefix for aref is 'method'.



75
76
77
# File 'lib/rdoc/code_object/any_method.rb', line 75

def aref_prefix
  'method'
end

#arglists ⇒ Object

The call_seq or the param_seq with method name, if there is no call_seq.

Use this for displaying a method's argument lists.



84
85
86
87
88
89
90
# File 'lib/rdoc/code_object/any_method.rb', line 84

def arglists
  if @call_seq
    @call_seq
  elsif @params
    "#{name}#{param_seq}"
  end
end

#call_seq ⇒ Object

Different ways to call this method



95
96
97
98
99
100
101
102
103
# File 'lib/rdoc/code_object/any_method.rb', line 95

def call_seq
  unless call_seq = _call_seq
    call_seq = is_alias_for._call_seq if is_alias_for
  end

  return unless call_seq

  deduplicate_call_seq(call_seq)
end

#call_seq=(call_seq) ⇒ Object

Sets the different ways you can call this method. If an empty call_seq is given nil is assumed.

See also #param_seq



111
112
113
114
115
# File 'lib/rdoc/code_object/any_method.rb', line 111

def call_seq=(call_seq)
  return if call_seq.nil? || call_seq.empty?

  @call_seq = call_seq
end

#has_call_seq? ⇒ Boolean

Whether the method has a call-seq.

Returns:

  • (Boolean)


120
121
122
# File 'lib/rdoc/code_object/any_method.rb', line 120

def has_call_seq?
  !!(@call_seq || is_alias_for&._call_seq)
end

#is_alias_for ⇒ Object

Loads is_alias_for from the internal name. Returns nil if the alias cannot be found.



128
129
130
131
132
133
134
135
136
137
138
139
140
141
# File 'lib/rdoc/code_object/any_method.rb', line 128

def is_alias_for # :nodoc:
  case @is_alias_for
  when MethodAttr
    @is_alias_for
  when Array
    return nil unless @store

    klass_name, singleton, method_name = @is_alias_for

    return nil unless klass = @store.find_class_or_module(klass_name)

    @is_alias_for = klass.find_method method_name, singleton
  end
end

#marshal_dump ⇒ Object

Dumps this AnyMethod for use by ri. See also #marshal_load



146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
# File 'lib/rdoc/code_object/any_method.rb', line 146

def marshal_dump
  aliases = @aliases.map do |a|
    [a.name, parse(a.comment)]
  end

  is_alias_for = [
    @is_alias_for.parent.full_name,
    @is_alias_for.singleton,
    @is_alias_for.name
  ] if @is_alias_for

  [ MARSHAL_VERSION,
    @name,
    full_name,
    @singleton,
    @visibility,
    parse(@comment),
    @call_seq,
    @block_params,
    aliases,
    @params,
    @file.relative_name,
    @calls_super,
    @parent.name,
    @parent.class,
    @section.title,
    is_alias_for,
    @type_signature_lines&.join("\n"),
  ]
end

#marshal_load(array) ⇒ Object

Loads this AnyMethod from array. For a loaded AnyMethod the following methods will return cached values:

  • #full_name
  • #parent_name


184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
# File 'lib/rdoc/code_object/any_method.rb', line 184

def marshal_load(array)
  initialize_visibility

  @dont_rename_initialize = nil
  @token_stream           = nil
  @aliases                = []
  @parent                 = nil
  @parent_name            = nil
  @parent_class           = nil
  @section                = nil
  @file                   = nil

  version        = array[0]
  @name          = array[1]
  @full_name     = array[2]
  @singleton     = array[3]
  @visibility    = array[4] || :public
  @comment       = Comment.from_document array[5]
  @call_seq      = array[6]
  @block_params  = array[7]
  #                      8 handled below
  @params        = array[9]
  #                      10 handled below
  @calls_super   = array[11]
  @parent_name   = array[12]
  @parent_title  = array[13]
  @section_title = array[14]
  @is_alias_for  = array[15]
  @type_signature_lines = array[16]&.split("\n")

  array[8].each do |new_name, document|
    add_alias Alias.new(@name, new_name, Comment.from_document(document), singleton: @singleton)
  end

  @parent_name ||= if @full_name =~ /#/
                     $`
                   else
                     name = @full_name.split('::')
                     name.pop
                     name.join '::'
                   end

  @file = TopLevel.new array[10] if version > 0
end

#name ⇒ Object

Method name

If the method has no assigned name, it extracts it from #call_seq.



234
235
236
237
238
239
240
241
# File 'lib/rdoc/code_object/any_method.rb', line 234

def name
  return @name if @name

  @name =
    @call_seq[/^.*?\.(\w+)/, 1] ||
    @call_seq[/^.*?(\w+)/, 1] ||
    @call_seq if @call_seq
end

#param_list ⇒ Object

A list of this method's method and yield parameters. call-seq params are preferred over parsed method and block params.



247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
# File 'lib/rdoc/code_object/any_method.rb', line 247

def param_list
  if @call_seq
    params = @call_seq.split("\n").last
    params = params.sub(/.*?\((.*)\)/, '\1')
    params = params.sub(/(\{|do)\s*\|([^|]*)\|.*/, ',\2')
  elsif @params
    params = @params.sub(/\((.*)\)/, '\1')

    params << ",#{@block_params}" if @block_params
  elsif @block_params
    params = @block_params
  else
    return []
  end

  if @block_params
    # If this method has explicit block parameters, remove any explicit
    # &block
    params = params.sub(/,?\s*&\w+/, '')
  else
    params = params.sub(/\&(\w+)/, '\1')
  end

  params = params.gsub(/\s+/, '').split(',').reject(&:empty?)

  params.map { |param| param.sub(/=.*/, '') }
end

#param_seq ⇒ Object

Pretty parameter list for this method. If the method's parameters were given by call-seq it is preferred over the parsed values.



279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
# File 'lib/rdoc/code_object/any_method.rb', line 279

def param_seq
  if @call_seq
    params = @call_seq.split("\n").last
    params = params.sub(/[^( ]+/, '')
    params = params.sub(/(\|[^|]+\|)\s*\.\.\.\s*(end|\})/, '\1 \2')
  elsif @params
    params = @params.gsub(/\s*\#.*/, '')
    params = params.tr_s("\n ", " ")
    params = "(#{params})" unless params[0] == ?(
  else
    params = ''
  end

  if @block_params
    # If this method has explicit block parameters, remove any explicit
    # &block
    params = params.sub(/,?\s*&\w+/, '')

    block = @block_params.tr_s("\n ", " ")
    if block[0] == ?(
      block = block.sub(/^\(/, '').sub(/\)/, '')
    end
    params << " { |#{block}| ... }"
  end

  params
end

#skip_description? ⇒ Boolean

Whether to skip the method description, true for methods that have aliases with a call-seq that doesn't include the method name.

Returns:

  • (Boolean)


311
312
313
# File 'lib/rdoc/code_object/any_method.rb', line 311

def skip_description?
  has_call_seq? && call_seq.nil? && !!(is_alias_for || !aliases.empty?)
end

#store=(store) ⇒ Object

Sets the store for this method and its referenced code objects.



318
319
320
321
322
# File 'lib/rdoc/code_object/any_method.rb', line 318

def store=(store)
  super

  @file = @store.add_file @file.full_name if @file
end

#superclass_method ⇒ Object

For methods that super, find the superclass method that would be called.



327
328
329
330
331
332
333
334
335
336
337
338
339
# File 'lib/rdoc/code_object/any_method.rb', line 327

def superclass_method
  return unless @calls_super
  return @superclass_method if @superclass_method

  parent.each_ancestor do |ancestor|
    if method = ancestor.method_list.find { |m| m.name == @name }
      @superclass_method = method
      break
    end
  end

  @superclass_method
end

Creates an HTML link to the superclass method called by this method.



98
99
100
101
102
103
104
105
106
# File 'lib/rdoc/generator/markup.rb', line 98

def superclass_method_link
  target = superclass_method
  return unless target

  html_formatter = formatter
  name = target.full_name

  html_formatter.link name, html_formatter.convert_string(name)
end