Class: RDoc::ClassModule

Inherits:
Context show all
Defined in:
lib/rdoc/code_object/class_module.rb,
lib/rdoc/generator/markup.rb

Overview

ClassModule is the base class for objects representing either a class or a module.

Direct Known Subclasses

NormalClass, NormalModule, SingleClass

Constant Summary collapse

MARSHAL_VERSION =
1

RDoc 3.7

  • Added visibility, singleton and file to attributes
  • Added file to constants
  • Added file to includes
  • Added file to methods
2

RDoc 3.13

  • Added extends
3

RDoc 4.0

  • Added sections
  • Added in_files
  • Added parent name
  • Complete Constant dump
3

Constants inherited from Context

RDoc::Context::TOMDOC_TITLES, RDoc::Context::TOMDOC_TITLES_SORT, RDoc::Context::TYPES

Constants included from Text

Text::MARKUP_FORMAT, Text::SPACE_SEPARATED_LETTER_CLASS

Instance Attribute Summary collapse

Attributes inherited from Context

#aliases, #attributes, #block_params, #constants, #constants_hash, #current_section, #extends, #external_aliases, #in_files, #includes, #method_list, #methods_hash, #name, #params, #requires, #temporary_section, #unmatched_alias_lists

Attributes inherited from CodeObject

#comment, #document_children, #document_self, #done_documenting, #file, #force_documentation, #line, #metadata, #mixin_from, #parent, #received_nodoc, #section, #store

Attributes included from Text

#language

Class Method Summary collapse

Instance Method Summary collapse

Methods inherited from Context

#<=>, #add, #add_alias, #add_attribute, #add_class, #add_class_or_module, #add_constant, #add_extend, #add_include, #add_method, #add_module, #add_module_alias, #add_module_by_normal_module, #add_require, #add_section, #add_to, #any_content, #child_name, #class_method_list, #classes, #classes_and_modules, #classes_hash, #display, #each_classmodule, #each_method, #each_section, #find_attribute, #find_attribute_named, #find_class_method_named, #find_constant_named, #find_enclosing_module_named, #find_external_alias, #find_external_alias_named, #find_instance_method_named, #find_local_symbol, #find_method, #find_method_named, #find_module_named, #find_or_create_constant_owner_for_path, #find_or_create_namespace_path, #find_symbol, #find_symbol_module, #fully_documented?, #get_module_named, #http_url, #initialize_methods_etc, #instance_methods, #methods_by_type, #methods_matching, #modules, #modules_hash, #record_location, #remove_from_documentation?, #remove_invisible, #remove_invisible_in, #resolve_aliases, #section_contents, #sections, #sections_hash, #set_constant_visibility_for, #set_current_section, #set_visibility_for, #sort_sections, #top_level, #upgrade_to_class

Methods inherited from CodeObject

#display?, #file_name, #full_name=, #ignore, #ignored?, #initialize_visibility, #options, #parent_name, #record_location, #start_doc, #stop_doc, #suppress, #suppressed?

Methods included from Generator::Markup

#aref_to, #as_href, #canonical_url, #cvs_url, #formatter

Methods included from Text

decode_legacy_label, expand_tabs, #flush_left, #markup, #normalize_comment, #snippet, #strip_hashes, #strip_newlines, #strip_stars, to_anchor, #wrap

Constructor Details

#initialize(name, superclass = nil) ⇒ ClassModule

Creates a new ClassModule with name with optional superclass

This is a constructor for subclasses, and must never be called directly.



120
121
122
123
124
125
126
127
128
# File 'lib/rdoc/code_object/class_module.rb', line 120

def initialize(name, superclass = nil)
  @constant_aliases = []
  @is_alias_for     = nil
  @name             = name
  @superclass       = superclass
  @comment_location = {} # Hash of { location => [comments] }

  super()
end

Instance Attribute Details

#comment_location ⇒ Object

A hash of { location => [comments] } documenting this class/module. Use #add_comment to add comments.

Ruby hashes maintain insertion order, so comments render in the order they were first added. Each location maps to an array of comments, allowing a class reopened in the same file to accumulate multiple comments.

Before marshalling:

  • location is an RDoc::TopLevel
  • comments are Strings

After unmarshalling:

  • location is a filename String
  • comments are RDoc::Markup::Documents


49
50
51
# File 'lib/rdoc/code_object/class_module.rb', line 49

def comment_location
  @comment_location
end

#constant_aliases ⇒ Object

Constants that are aliases for this class or module



31
32
33
# File 'lib/rdoc/code_object/class_module.rb', line 31

def constant_aliases
  @constant_aliases
end

#is_alias_for ⇒ Object

Class or module this constant is an alias for



54
55
56
# File 'lib/rdoc/code_object/class_module.rb', line 54

def is_alias_for
  @is_alias_for
end

Class Method Details

.from_module(class_type, mod) ⇒ Object

Return a RDoc::ClassModule of class class_type that is a copy of module module. Used to promote modules to classes.

TODO move to RDoc::NormalClass (I think)



62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
# File 'lib/rdoc/code_object/class_module.rb', line 62

def self.from_module(class_type, mod)
  klass = class_type.new mod.name

  mod.comment_location.each do |location, comments|
    comments.each { |comment| klass.add_comment comment, location }
  end

  klass.parent = mod.parent
  klass.section = mod.section

  klass.attributes.concat mod.attributes
  klass.method_list.concat mod.method_list
  klass.aliases.concat mod.aliases
  klass.external_aliases.concat mod.external_aliases
  klass.constants.concat mod.constants
  klass.includes.concat mod.includes
  klass.extends.concat mod.extends

  klass.methods_hash.update mod.methods_hash
  klass.constants_hash.update mod.constants_hash

  klass.current_section = mod.current_section
  klass.in_files.concat mod.in_files
  klass.sections.concat mod.sections
  klass.unmatched_alias_lists = mod.unmatched_alias_lists
  klass.current_section = mod.current_section
  klass.classes_hash.update mod.classes_hash
  klass.modules_hash.update mod.modules_hash
  klass..update mod.

  klass.document_self = mod.received_nodoc ? nil : mod.document_self
  klass.document_children = mod.document_children
  klass.force_documentation = mod.force_documentation
  klass.done_documenting = mod.done_documenting

  # update the parent of all children

  (klass.attributes +
   klass.method_list +
   klass.aliases +
   klass.external_aliases +
   klass.constants +
   klass.includes +
   klass.extends +
   klass.classes +
   klass.modules).each do |obj|
    obj.parent = klass
    obj.full_name = nil
  end

  klass
end

Instance Method Details

#add_comment(comment, location) ⇒ Object

Adds comment to this ClassModule's list of comments at location. This method is preferred over #comment= since it allows ri data to be updated across multiple runs.



135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
# File 'lib/rdoc/code_object/class_module.rb', line 135

def add_comment(comment, location)
  return unless document_self

  original = comment

  comment = case comment
            when Comment
              comment.normalize
            else
              normalize_comment comment
            end

  (@comment_location[location] ||= []) << comment

  self.comment = original
end

#add_things(my_things, other_things) ⇒ Object

:nodoc:



152
153
154
155
156
157
158
159
160
161
# File 'lib/rdoc/code_object/class_module.rb', line 152

def add_things(my_things, other_things) # :nodoc:
  other_things.each do |group, things|
    my_things[group].each { |thing| yield false, thing } if
      my_things.include? group

    things.each do |thing|
      yield true, thing
    end
  end
end

#ancestors ⇒ Object Also known as: direct_ancestors

Ancestors list for this ClassModule: the list of included modules (classes will add their superclass if any).

Returns the included classes or modules, not the includes themselves. The returned values are either String or RDoc::NormalModule instances (see RDoc::Include#module).

The values are returned in reverse order of their inclusion, which is the order suitable for searching methods/attributes in the ancestors. The superclass, if any, comes last.



175
176
177
# File 'lib/rdoc/code_object/class_module.rb', line 175

def ancestors
  included_ancestors
end

#aref ⇒ Object

HTML fragment reference for this module or class using GitHub-style

anchor format (lowercase,

replaced with -).

Examples:

Foo      -> class-foo
Foo::Bar -> class-foo-bar


195
196
197
# File 'lib/rdoc/code_object/class_module.rb', line 195

def aref
  "#{aref_prefix}-#{full_name.downcase.gsub('::', '-')}"
end

#aref_prefix ⇒ Object

:nodoc:

Raises:

  • (NotImplementedError)


183
184
185
# File 'lib/rdoc/code_object/class_module.rb', line 183

def aref_prefix # :nodoc:
  raise NotImplementedError, "missing aref_prefix for #{self.class}"
end

#clear_comment ⇒ Object

Clears the comment. Used by the Ruby parser.



219
220
221
# File 'lib/rdoc/code_object/class_module.rb', line 219

def clear_comment
  @comment = ''
end

#comment=(comment) ⇒ Object

This method is deprecated, use #add_comment instead.

Appends comment to the current comment, but separated by a rule. Works more like +=.



229
230
231
232
233
234
235
236
237
238
239
240
# File 'lib/rdoc/code_object/class_module.rb', line 229

def comment=(comment) # :nodoc:
  comment = case comment
            when Comment
              comment.normalize
            else
              normalize_comment comment
            end

  comment = "#{@comment.to_s}\n---\n#{comment.to_s}" unless @comment.empty?

  super comment
end

#complete(min_visibility) ⇒ Object

Prepares this ClassModule for use by a generator.

See RDoc::Store#complete



247
248
249
250
251
252
253
254
# File 'lib/rdoc/code_object/class_module.rb', line 247

def complete(min_visibility)
  update_aliases
  remove_nodoc_children
  embed_mixins
  update_includes
  update_extends
  remove_invisible min_visibility
end

#description ⇒ Object

Handy wrapper for marking up this class or module's comment



179
180
181
# File 'lib/rdoc/generator/markup.rb', line 179

def description
  markup @comment_location
end

#document_self_or_methods ⇒ Object

Does this ClassModule or any of its methods have document_self set?



259
260
261
# File 'lib/rdoc/code_object/class_module.rb', line 259

def document_self_or_methods
  document_self || method_list.any?{ |m| m.document_self }
end

#documented? ⇒ Boolean

Does this class or module have a comment with content or is #received_nodoc true?

Returns:

  • (Boolean)


267
268
269
270
271
# File 'lib/rdoc/code_object/class_module.rb', line 267

def documented?
  return true if @received_nodoc
  return false if @comment_location.empty?
  @comment_location.each_value.any? { |comments| comments.any? { |c| not c.empty? } }
end

#each_ancestor ⇒ Object

Iterates the ancestors of this class or module for which an RDoc::ClassModule exists.



277
278
279
280
281
282
283
284
285
# File 'lib/rdoc/code_object/class_module.rb', line 277

def each_ancestor # :yields: module
  return enum_for __method__ unless block_given?

  ancestors.each do |mod|
    next if String === mod
    next if self == mod
    yield mod
  end
end

#embed_mixins ⇒ Object



938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
# File 'lib/rdoc/code_object/class_module.rb', line 938

def embed_mixins
  return unless options.embed_mixins

  includes.each do |include|
    next if String === include.module
    include.module.method_list.each do |code_object|
      add_method(prepare_to_embed(code_object))
    end
    include.module.constants.each do |code_object|
      add_constant(prepare_to_embed(code_object))
    end
    include.module.attributes.each do |code_object|
      add_attribute(prepare_to_embed(code_object))
    end
  end

  extends.each do |ext|
    next if String === ext.module
    ext.module.method_list.each do |code_object|
      add_method(prepare_to_embed(code_object, true))
    end
    ext.module.attributes.each do |code_object|
      add_attribute(prepare_to_embed(code_object, true))
    end
  end
end

#find_ancestor_local_symbol(symbol) ⇒ Object

Looks for a symbol in the #ancestors. See Context#find_local_symbol.



290
291
292
293
294
295
296
297
# File 'lib/rdoc/code_object/class_module.rb', line 290

def find_ancestor_local_symbol(symbol)
  each_ancestor do |m|
    res = m.find_local_symbol(symbol)
    return res if res
  end

  nil
end

#find_class_named(name) ⇒ Object

Finds a class or module with name in this namespace or its descendants



302
303
304
305
306
307
308
309
310
# File 'lib/rdoc/code_object/class_module.rb', line 302

def find_class_named(name)
  return self if full_name == name
  return self if @name == name

  @classes.values.find do |klass|
    next if klass == self
    klass.find_class_named name
  end
end

#full_name ⇒ Object

Return the fully qualified name of this class or module



315
316
317
318
319
320
321
# File 'lib/rdoc/code_object/class_module.rb', line 315

def full_name
  @full_name ||= if ClassModule === parent
                   "#{parent.full_name}::#{@name}"
                 else
                   @name
                 end
end

#fully_qualified_nesting_namespaces ⇒ Object

Return array of fully qualified nesting namespaces.

For example, if full_name is A::B::C, this method returns ["A", "A::B", "A::B::C"]



335
336
337
338
339
340
# File 'lib/rdoc/code_object/class_module.rb', line 335

def fully_qualified_nesting_namespaces
  return nesting_namespaces if nesting_namespaces.length < 2
  @fqns ||= nesting_namespaces.inject([]) do |list, n|
    list << (list.empty? ? n : "#{list.last}::#{n}")
  end
end

#included_ancestors ⇒ Object

:nodoc:



179
180
181
# File 'lib/rdoc/code_object/class_module.rb', line 179

def included_ancestors # :nodoc:
  includes.map { |i| i.module }.reverse
end

#legacy_aref ⇒ Object

Legacy HTML fragment reference for backward compatibility. Returns the old RDoc-style anchor format.

Examples:

Foo      -> class-Foo
Foo::Bar -> class-Foo::Bar


207
208
209
# File 'lib/rdoc/code_object/class_module.rb', line 207

def legacy_aref
  "#{aref_prefix}-#{full_name}"
end

#marshal_dump ⇒ Object

TODO: filter included items by #display?



345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
# File 'lib/rdoc/code_object/class_module.rb', line 345

def marshal_dump # :nodoc:
  attrs = attributes.sort.map do |attr|
    next unless attr.display?
    [ attr.name, attr.rw,
      attr.visibility, attr.singleton, attr.file_name,
    ]
  end.compact

  method_types = methods_by_type.map do |type, visibilities|
    visibilities = visibilities.map do |visibility, methods|
      method_names = methods.map do |method|
        next unless method.display?
        [method.name, method.file_name]
      end.compact

      [visibility, method_names.uniq]
    end

    [type, visibilities]
  end

  [ MARSHAL_VERSION,
    @name,
    full_name,
    @superclass,
    parse(@comment_location),
    attrs,
    constants.select { |constant| constant.display? },
    includes.map do |incl|
      next unless incl.display?
      [incl.name, parse(incl.comment), incl.file_name]
    end.compact,
    method_types,
    extends.map do |ext|
      next unless ext.display?
      [ext.name, parse(ext.comment), ext.file_name]
    end.compact,
    @sections.values,
    @in_files.map do |tl|
      tl.relative_name
    end,
    parent.full_name,
    parent.class,
  ]
end

#marshal_load(array) ⇒ Object

:nodoc:



391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
# File 'lib/rdoc/code_object/class_module.rb', line 391

def marshal_load(array) # :nodoc:
  initialize_visibility
  initialize_methods_etc
  @current_section   = nil
  @document_self     = true
  @done_documenting  = false
  @parent            = nil
  @temporary_section = nil
  @classes           = {}
  @modules           = {}

  @name       = array[1]
  @full_name  = array[2]
  @superclass = array[3]
  document    = array[4]

  @comment    = Comment.from_document document

  @comment_location = if document.parts.first.is_a?(Markup::Document)
                        document.parts.group_by(&:file)
                      else
                        { document.file => [document] }
                      end

  array[5].each do |name, rw, visibility, singleton, file|
    singleton  ||= false
    visibility ||= :public

    attr = Attr.new name, rw, nil, singleton: singleton

    add_attribute attr
    attr.visibility = visibility
    attr.record_location TopLevel.new file
  end

  array[6].each do |constant, document, file|
    case constant
    when Constant
      add_constant constant
    else
      constant = add_constant Constant.new(constant, nil, Comment.from_document(document))
      constant.record_location TopLevel.new file
    end
  end

  array[7].each do |name, document, file|
    incl = add_include Include.new(name, Comment.from_document(document))
    incl.record_location TopLevel.new file
  end

  array[8].each do |type, visibilities|
    visibilities.each do |visibility, methods|
      methods.each do |name, file|
        method = AnyMethod.new name, singleton: type == 'class'
        method.record_location TopLevel.new file
        method.visibility = visibility
        add_method method
      end
    end
  end

  array[9].each do |name, document, file|
    ext = add_extend Extend.new(name, Comment.from_document(document))
    ext.record_location TopLevel.new file
  end if array[9] # Support Marshal version 1

  sections = (array[10] || []).map do |section|
    [section.title, section]
  end

  @sections = Hash[*sections.flatten]
  @current_section = add_section nil

  @in_files = []

  (array[11] || []).each do |filename|
    record_location TopLevel.new filename
  end

  @parent_name  = array[12]
  @parent_class = array[13]
end

#merge(class_module) ⇒ Object

Merges class_module into this ClassModule.

The data in class_module is preferred over the receiver.



479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
# File 'lib/rdoc/code_object/class_module.rb', line 479

def merge(class_module)
  @parent      = class_module.parent
  @parent_name = class_module.parent_name

  other_document = parse class_module.comment_location

  if other_document
    document = parse @comment_location

    document = document.merge other_document

    @comment = Comment.from_document(document)

    @comment_location = if document.parts.first.is_a?(Markup::Document)
                          document.parts.group_by(&:file)
                        else
                          { document.file => [document] }
                        end
  end

  cm = class_module
  other_files = cm.in_files

  merge_collections attributes, cm.attributes, other_files do |add, attr|
    if add
      add_attribute attr
    else
      @attributes.delete attr
      @methods_hash.delete attr.pretty_name
    end
  end

  merge_collections constants, cm.constants, other_files do |add, const|
    if add
      add_constant const
    else
      @constants.delete const
      @constants_hash.delete const.name
    end
  end

  merge_collections includes, cm.includes, other_files do |add, incl|
    if add
      add_include incl
    else
      @includes.delete incl
    end
  end

  @includes.uniq! # clean up

  merge_collections extends, cm.extends, other_files do |add, ext|
    if add
      add_extend ext
    else
      @extends.delete ext
    end
  end

  @extends.uniq! # clean up

  merge_collections method_list, cm.method_list, other_files do |add, meth|
    if add
      add_method meth
    else
      @method_list.delete meth
      @methods_hash.delete meth.pretty_name
    end
  end

  merge_sections cm

  self
end

#merge_collections(mine, other, other_files, &block) ⇒ Object

Merges collection mine with other preferring other. other_files is used to help determine which items should be deleted.

Yields whether the item should be added or removed (true or false) and the item to be added or removed.

merge_collections things, other.things, other.in_files do |add, thing|
if add
  # add the thing
else
  # remove the thing
end
end


569
570
571
572
573
574
575
# File 'lib/rdoc/code_object/class_module.rb', line 569

def merge_collections(mine, other, other_files, &block) # :nodoc:
  my_things    = mine. group_by { |thing| thing.file }
  other_things = other.group_by { |thing| thing.file }

  remove_things my_things, other_files,  &block
  add_things    my_things, other_things, &block
end

#merge_sections(cm) ⇒ Object

Merges the comments in this ClassModule with the comments in the other ClassModule cm.



581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
# File 'lib/rdoc/code_object/class_module.rb', line 581

def merge_sections(cm) # :nodoc:
  my_sections    =    sections.group_by { |section| section.title }
  other_sections = cm.sections.group_by { |section| section.title }

  other_files = cm.in_files

  remove_things my_sections, other_files do |_, section|
    @sections.delete section.title
  end

  other_sections.each do |group, sections|
    if my_sections.include? group
      my_sections[group].each do |my_section|
        other_section = cm.sections_hash[group]

        my_comments    = my_section.comments
        other_comments = other_section.comments

        other_files = other_section.in_files

        merge_collections my_comments, other_comments, other_files do |add, comment|
          if add
            my_section.add_comment comment
          else
            my_section.remove_comment comment
          end
        end
      end
    else
      sections.each do |section|
        add_section group, section.comments
      end
    end
  end
end

#module? ⇒ Boolean

Does this object represent a module?

Returns:

  • (Boolean)


620
621
622
# File 'lib/rdoc/code_object/class_module.rb', line 620

def module?
  false
end

#name=(new_name) ⇒ Object

Allows overriding the initial name.

Used for modules and classes that are constant aliases.



629
630
631
# File 'lib/rdoc/code_object/class_module.rb', line 629

def name=(new_name)
  @name = new_name
end

#name_for_path ⇒ Object

Name to use to generate the url: modules and classes that are aliases for another module or class return the name of the latter.



676
677
678
# File 'lib/rdoc/code_object/class_module.rb', line 676

def name_for_path
  is_alias_for ? is_alias_for.name_for_path : full_name
end

#nesting_namespaces ⇒ Object

Return array of full_name splitted by ::.



326
327
328
# File 'lib/rdoc/code_object/class_module.rb', line 326

def nesting_namespaces
  @namespaces ||= full_name.split("::").reject(&:empty?)
end

#non_aliases ⇒ Object

Returns the classes and modules that are not constants aliasing another class or module. For use by formatters only (caches its result).



685
686
687
# File 'lib/rdoc/code_object/class_module.rb', line 685

def non_aliases
  @non_aliases ||= classes_and_modules.reject { |cm| cm.is_alias_for }
end

#parse(comment_location) ⇒ Object

Parses comment_location into an RDoc::Markup::Document composed of multiple RDoc::Markup::Documents with their file set.



637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
# File 'lib/rdoc/code_object/class_module.rb', line 637

def parse(comment_location)
  case comment_location
  when String
    super
  when Hash
    docs = comment_location.flat_map do |location, comments|
      comments.map do |comment|
        doc = super comment
        doc.file = location
        doc
      end
    end

    Markup::Document.new(*docs)
  when Comment
    doc = super comment_location.text, comment_location.format
    doc.file = comment_location.location
    doc
  when Markup::Document
    return comment_location
  else
    raise ArgumentError, "unknown comment class #{comment_location.class}"
  end
end

#path ⇒ Object

Path to this class or module for use with HTML generator output.



665
666
667
668
669
# File 'lib/rdoc/code_object/class_module.rb', line 665

def path
  prefix = options.class_module_path_prefix
  return http_url unless prefix
  File.join(prefix, http_url)
end

#rebuild_comment_from_location ⇒ Object

Rebuilds @comment from the current @comment_location entries, skipping any empty placeholders.



755
756
757
758
759
760
761
# File 'lib/rdoc/code_object/class_module.rb', line 755

def rebuild_comment_from_location
  texts = @comment_location.each_value.flat_map { |comments|
    comments.filter_map { |c| c.to_s unless c.empty? }
  }
  merged = texts.join("\n---\n")
  @comment = merged.empty? ? '' : Comment.new(merged)
end

#remove_nodoc_children ⇒ Object

Updates the child modules or classes of class/module parent by deleting the ones that have been removed from the documentation.

parent_hash is either parent.modules_hash or parent.classes_hash and all_hash is ::all_modules_hash or ::all_classes_hash.



697
698
699
700
701
702
703
704
705
706
707
708
709
# File 'lib/rdoc/code_object/class_module.rb', line 697

def remove_nodoc_children
  prefix = self.full_name + '::'

  modules_hash.each_key do |name|
    full_name = prefix + name
    modules_hash.delete name unless @store.modules_hash[full_name]
  end

  classes_hash.each_key do |name|
    full_name = prefix + name
    classes_hash.delete name unless @store.classes_hash[full_name]
  end
end

#remove_things(my_things, other_files) ⇒ Object

:nodoc:



711
712
713
714
715
716
717
718
719
720
721
# File 'lib/rdoc/code_object/class_module.rb', line 711

def remove_things(my_things, other_files) # :nodoc:
  my_things.delete_if do |file, things|
    next false unless other_files.include? file

    things.each do |thing|
      yield false, thing
    end

    true
  end
end

#search_record ⇒ Object

Search record used by RDoc::Generator::JsonIndex

TODO: Remove this method after dropping the darkfish theme and JsonIndex generator. Use #search_snippet instead for getting documentation snippets.



729
730
731
732
733
734
735
736
737
738
739
# File 'lib/rdoc/code_object/class_module.rb', line 729

def search_record
  [
    name,
    full_name,
    full_name,
    '',
    path,
    '',
    snippet(@comment_location),
  ]
end

#search_snippet ⇒ Object

Returns an HTML snippet of the first comment for search results.



744
745
746
747
748
749
# File 'lib/rdoc/code_object/class_module.rb', line 744

def search_snippet
  first_comment = @comment_location.each_value.first&.first
  return '' unless first_comment && !first_comment.empty?

  snippet(first_comment)
end

#store=(store) ⇒ Object

Sets the store for this class or module and its contained code objects.



766
767
768
769
770
771
772
773
774
# File 'lib/rdoc/code_object/class_module.rb', line 766

def store=(store)
  super

  @attributes .each do |attr|  attr.store  = store end
  @constants  .each do |const| const.store = store end
  @includes   .each do |incl|  incl.store  = store end
  @extends    .each do |ext|   ext.store   = store end
  @method_list.each do |meth|  meth.store  = store end
end

#super_classes ⇒ Object

Get all super classes of this class in an array. The last element might be a string if the name is unknown.



809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
# File 'lib/rdoc/code_object/class_module.rb', line 809

def super_classes
  result = []
  # Degenerate input can produce a cyclic superclass chain
  visited = [full_name]
  parent = self
  while parent = parent.superclass
    if parent.is_a?(String)
      result << parent
      break
    end
    break if visited.include?(parent.full_name)
    visited << parent.full_name
    result << parent
  end
  result
end

#superclass ⇒ Object

Get the superclass of this class. Attempts to retrieve the superclass object, returns the name if it is not known.



780
781
782
# File 'lib/rdoc/code_object/class_module.rb', line 780

def superclass
  @store.find_class_named(@superclass) || @superclass
end

#superclass=(superclass) ⇒ Object

Set the superclass of this class to superclass

where superclass is one of:

  • nil
  • a String containing the full name of the superclass
  • the RDoc::ClassModule representing the superclass

Raises:

  • (NoMethodError)


793
794
795
796
797
798
799
800
801
802
803
# File 'lib/rdoc/code_object/class_module.rb', line 793

def superclass=(superclass)
  raise NoMethodError, "#{full_name} is a module" if module?
  case superclass
  when ClassModule
    @superclass = superclass.full_name
  when nil, String
    @superclass = superclass
  else
    raise TypeError, "superclass must be a String or RDoc::ClassModule, not #{superclass.class}"
  end
end

#to_s ⇒ Object

:nodoc:



826
827
828
829
830
831
832
# File 'lib/rdoc/code_object/class_module.rb', line 826

def to_s # :nodoc:
  if is_alias_for
    "#{self.class.name} #{self.full_name} -> #{is_alias_for}"
  else
    super
  end
end

#type ⇒ Object

'module' or 'class'



837
838
839
# File 'lib/rdoc/code_object/class_module.rb', line 837

def type
  module? ? 'module' : 'class'
end

#update_aliases ⇒ Object

Updates the child modules & classes by replacing the ones that are aliases through a constant.

The aliased module/class is replaced in the children and in RDoc::Store#modules_hash or RDoc::Store#classes_hash by a copy that has RDoc::ClassModule#is_alias_for set to the aliased module/class, and this copy is added to #aliases of the aliased module/class.

Formatters can use the #non_aliases method to retrieve children that are not aliases, for instance to list the namespace content, since the aliased modules are included in the constants of the class/module, that are listed separately.



856
857
858
859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
# File 'lib/rdoc/code_object/class_module.rb', line 856

def update_aliases
  constants.each do |const|
    cm = const.is_alias_for
    cm ||= const.resolved_alias_target if const.is_a?(Constant)
    next unless cm

    # Resolve chained aliases (A = B = C) to the real class/module.
    cm = @store.find_class_or_module(cm.full_name) || cm
    while (target = cm.is_alias_for)
      cm = target
    end

    cm_alias = cm.dup
    cm_alias.name = const.name

    if full_name == 'Object'
      # Don't move top-level aliases under Object, they look ugly there
      cm_alias.parent = top_level
    else
      cm_alias.parent = self
    end
    cm_alias.full_name = nil # force update for new parent

    # Don't clobber a real (non-alias) class/module already living at this
    # name. Mirrors the BasicObject = BlankSlate guard in
    # Context#add_module_alias. Existing alias copies (set by
    # add_module_alias or a previous update_aliases pass) carry is_alias_for,
    # so they're still overwritable here.
    existing = @store.find_class_or_module(cm_alias.full_name)
    next if existing && !existing.is_alias_for

    # Persist a lazy-resolved target so Stats#report_constants and
    # Constant#marshal_dump observe the alias relationship. Skipped
    # aliases (above) intentionally leave the constant unmarked.
    const.is_alias_for ||= cm

    cm_alias.aliases.clear
    cm_alias.is_alias_for = cm

    if cm.module?
      @store.modules_hash[cm_alias.full_name] = cm_alias
      modules_hash[const.name] = cm_alias
    else
      @store.classes_hash[cm_alias.full_name] = cm_alias
      classes_hash[const.name] = cm_alias
    end

    cm.aliases << cm_alias
  end
end

#update_extends ⇒ Object

Deletes from #extends those whose module has been removed from the documentation.

FIXME: like update_includes, extends are not reliably removed



928
929
930
931
932
933
934
935
936
# File 'lib/rdoc/code_object/class_module.rb', line 928

def update_extends
  extends.reject! do |ext|
    mod = ext.module

    !(String === mod) && @store.modules_hash[mod.full_name].nil?
  end

  extends.uniq!
end

#update_includes ⇒ Object

Deletes from #includes those whose module has been removed from the documentation.

FIXME: includes are not reliably removed, see _possible_bug test case



913
914
915
916
917
918
919
920
# File 'lib/rdoc/code_object/class_module.rb', line 913

def update_includes
  includes.reject! do |include|
    mod = include.module
    !(String === mod) && @store.modules_hash[mod.full_name].nil?
  end

  includes.uniq!
end