Class: RDoc::Generator::Darkfish

Inherits:
Object
  • Object
show all
Includes:
ERB::Util
Defined in:
lib/rdoc/generator/darkfish.rb

Overview

Darkfish RDoc HTML Generator

$Id: darkfish.rb 52 2009-01-07 02:08:11Z deveiant $

Author/s

Contributors

License

Copyright (c) 2007, 2008, Michael Granger. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

  • Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

  • Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

  • Neither the name of the author/s, nor the names of the project's contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Attributions

Darkfish uses the Silk Icons set by Mark James.

Constant Summary collapse

BUILTIN_STYLE_ITEMS =

Stylesheets, fonts, etc. that are included in RDoc.

%w[
    css/fonts.css
    fonts/Lato-Light.ttf
    fonts/Lato-LightItalic.ttf
    fonts/Lato-Regular.ttf
    fonts/Lato-RegularItalic.ttf
    fonts/SourceCodePro-Bold.ttf
    fonts/SourceCodePro-Regular.ttf
    css/rdoc.css
]
DESCRIPTION =

Description of this generator

'HTML generator, written by Michael Granger'
ParagraphExcerptRegexpOther =

:stopdoc:

%r[\b\w[^./:]++\.]
ParagraphExcerptRegexpUnicode =

use \p/\Pletter instead of \w/\W in Unicode

%r[\b\p{letter}[^./:]++\.]

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(store, options) ⇒ Darkfish

Initialize a few instance variables before we start



150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
# File 'lib/rdoc/generator/darkfish.rb', line 150

def initialize(store, options)
  @store   = store
  @options = options

  @asset_rel_path = ''
  @base_dir       = Pathname.pwd.expand_path
  @dry_run        = @options.dry_run
  @file_output    = true
  @template_dir   = Pathname.new options.template_dir
  @template_cache = {}

  @classes = nil
  @context = nil
  @files   = nil
  @methods = nil
  @modsort = nil

  @json_index = Generator::JsonIndex.new self, options
end

Instance Attribute Details

#asset_rel_path ⇒ Object

The relative path to style sheets and javascript. By default this is set the same as the rel_prefix.



87
88
89
# File 'lib/rdoc/generator/darkfish.rb', line 87

def asset_rel_path
  @asset_rel_path
end

#base_dir ⇒ Object (readonly)

The path to generate files into, combined with --op from the options for a full path.



93
94
95
# File 'lib/rdoc/generator/darkfish.rb', line 93

def base_dir
  @base_dir
end

#classes ⇒ Object (readonly)

Classes and modules to be used by this generator, not necessarily displayed. See also #modsort



99
100
101
# File 'lib/rdoc/generator/darkfish.rb', line 99

def classes
  @classes
end

#dry_run ⇒ Object

No files will be written when dry_run is true.



104
105
106
# File 'lib/rdoc/generator/darkfish.rb', line 104

def dry_run
  @dry_run
end

#file_output ⇒ Object

When false the generate methods return a String instead of writing to a file. The default is true.



110
111
112
# File 'lib/rdoc/generator/darkfish.rb', line 110

def file_output
  @file_output
end

#files ⇒ Object (readonly)

Files to be displayed by this generator



115
116
117
# File 'lib/rdoc/generator/darkfish.rb', line 115

def files
  @files
end

#json_index ⇒ Object (readonly)

The JSON index generator for this Darkfish generator



120
121
122
# File 'lib/rdoc/generator/darkfish.rb', line 120

def json_index
  @json_index
end

#methods ⇒ Object (readonly)

Methods to be displayed by this generator



125
126
127
# File 'lib/rdoc/generator/darkfish.rb', line 125

def methods
  @methods
end

#modsort ⇒ Object (readonly)

Sorted list of classes and modules to be displayed by this generator



130
131
132
# File 'lib/rdoc/generator/darkfish.rb', line 130

def modsort
  @modsort
end

#outputdir ⇒ Object (readonly)

The output directory



145
146
147
# File 'lib/rdoc/generator/darkfish.rb', line 145

def outputdir
  @outputdir
end

#store ⇒ Object (readonly)

The RDoc::Store that is the source of the generated content



135
136
137
# File 'lib/rdoc/generator/darkfish.rb', line 135

def store
  @store
end

#template_dir ⇒ Object (readonly)

The directory where the template files live



140
141
142
# File 'lib/rdoc/generator/darkfish.rb', line 140

def template_dir
  @template_dir
end

Instance Method Details

#assemble_template(body_file) ⇒ Object

Creates a template from its components and the body_file.

For backwards compatibility, if body_file contains "<html" the body is used directly.



605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
# File 'lib/rdoc/generator/darkfish.rb', line 605

def assemble_template(body_file)
  body = body_file.read(encoding: template_encoding)
  return body if body =~ /<html/

  head_file = @template_dir + '_head.rhtml'

  <<-TEMPLATE
<!DOCTYPE html>

<html lang="#{@options.locale&.name || 'en'}">
<head>
#{head_file.read(encoding: template_encoding)}

#{body}
    TEMPLATE
end

#copy_static ⇒ Object

Copies static files from the static_path into the output directory



242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
# File 'lib/rdoc/generator/darkfish.rb', line 242

def copy_static
  return if @options.static_path.empty?

  fu_options = { :verbose => $DEBUG_RDOC, :noop => @dry_run }

  @options.static_path.each do |path|
    unless File.directory? path
      FileUtils.install path, @outputdir, **fu_options.merge(:mode => 0644)
      next
    end

    Dir.chdir path do
      Dir[File.join('**', '*')].each do |entry|
        dest_file = @outputdir + entry

        if File.directory? entry
          FileUtils.mkdir_p entry, **fu_options
        else
          FileUtils.install entry, dest_file, **fu_options.merge(:mode => 0644)
        end
      end
    end
  end
end

#debug_msg(*msg) ⇒ Object

Output progress information if debugging is enabled



173
174
175
176
# File 'lib/rdoc/generator/darkfish.rb', line 173

def debug_msg(*msg)
  return unless $DEBUG_RDOC
  $stderr.puts(*msg)
end

#excerpt(comment) ⇒ Object

Returns an excerpt of the comment for usage in meta description tags



720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
# File 'lib/rdoc/generator/darkfish.rb', line 720

def excerpt(comment)
  text = case comment
  when Comment
    comment.text
  else
    comment
  end

  # Match from a capital letter to the first period, discarding any links, so
  # that we don't end up matching badges in the README
  pattern = ParagraphExcerptRegexpUnicode
  begin
    first_paragraph_match = text.match(pattern)
  rescue ::Encoding::CompatibilityError
    # The doc is non-ASCII text and encoded in other than Unicode base encodings.
    raise if pattern == ParagraphExcerptRegexpOther
    pattern = ParagraphExcerptRegexpOther
    retry
  end
  return text[0...150].tr_s("\n", " ").squeeze(" ") unless first_paragraph_match

  extracted_text = first_paragraph_match[0]
  second_paragraph = text.match(pattern, first_paragraph_match.end(0))
  extracted_text << " " << second_paragraph[0] if second_paragraph

  extracted_text[0...150].tr_s("\n", " ").squeeze(" ")
end

#gen_sub_directories ⇒ Object

Create the directories the generated docs will live in if they don't already exist.



182
183
184
# File 'lib/rdoc/generator/darkfish.rb', line 182

def gen_sub_directories
  @outputdir.mkpath
end

#generate ⇒ Object

Build the initial indices and output objects based on an array of TopLevel objects containing the extracted information.



216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
# File 'lib/rdoc/generator/darkfish.rb', line 216

def generate
  warn "The Darkfish template is deprecated and will be removed in version 9.0."

  setup

  write_style_sheet
  generate_index
  generate_class_files
  generate_file_files
  generate_table_of_contents
  @json_index.generate
  @json_index.generate_gzipped

  copy_static

rescue => e
  debug_msg "%s: %s\n  %s" % [
    e.class.name, e.message, e.backtrace.join("\n  ")
  ]

  raise
end

#generate_ancestor_list(ancestors, klass) ⇒ Object



748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
# File 'lib/rdoc/generator/darkfish.rb', line 748

def generate_ancestor_list(ancestors, klass)
  return '' if ancestors.empty?

  ancestor = ancestors.shift
  content = +'<ul><li>'

  if ancestor.is_a?(NormalClass)
    content << "<a href=\"#{klass.aref_to ancestor.path}\">#{ancestor.full_name}</a>"
  else
    content << ancestor.to_s
  end

  # Recursively call the method for the remaining ancestors
  content << generate_ancestor_list(ancestors, klass)

  content << '</li></ul>'
end

#generate_class(klass, template_file = nil) ⇒ Object

Generates a class file for klass



315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
# File 'lib/rdoc/generator/darkfish.rb', line 315

def generate_class(klass, template_file = nil)
  # This is used to auto-collapse Pages section on class/module pages
  @inside_class_file = true
  current = klass

  template_file ||= @template_dir + 'class.rhtml'

  debug_msg "  working on %s (%s)" % [klass.full_name, klass.path]
  out_file   = @outputdir + klass.path
  rel_prefix = @outputdir.relative_path_from out_file.dirname
  search_index_rel_prefix = rel_prefix
  search_index_rel_prefix += @asset_rel_path if @file_output

  asset_rel_prefix = rel_prefix + @asset_rel_path

  breadcrumb = # used in templates
  breadcrumb = generate_nesting_namespaces_breadcrumb(current, rel_prefix)

  @title = "#{klass.type} #{klass.full_name} - #{@options.title}"

  debug_msg "  rendering #{out_file}"
  render_template template_file, out_file do |io|
    here = binding
    # suppress 1.9.3 warning
    here.local_variable_set(:asset_rel_prefix, asset_rel_prefix)
    here
  end
ensure
  @inside_class_file = false
end

#generate_class_files ⇒ Object

Generate a documentation file for each class and module



349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
# File 'lib/rdoc/generator/darkfish.rb', line 349

def generate_class_files
  template_file = @template_dir + 'class.rhtml'
  template_file = @template_dir + 'classpage.rhtml' unless
    template_file.exist?
  return unless template_file.exist?
  debug_msg "Generating class documentation in #{@outputdir}"

  current = nil

  # Document files are generated only for non-alias classes/modules
  @classes.reject(&:is_alias_for).each do |klass|

    current = klass

    generate_class klass, template_file
  end
rescue => e
  error = Error.new \
    "error generating #{current.path}: #{e.message} (#{e.class})"
  error.set_backtrace e.backtrace

  raise error
end

#generate_class_index_content(classes, rel_prefix) ⇒ Object



774
775
776
777
778
779
780
# File 'lib/rdoc/generator/darkfish.rb', line 774

def generate_class_index_content(classes, rel_prefix)
  grouped_classes = group_classes_by_namespace_for_sidebar(classes)
  return '' unless top = grouped_classes[nil]

  solo = top.one? { |klass| klass.display? }
  traverse_classes(top, grouped_classes, rel_prefix, solo)
end


766
767
768
769
770
771
772
# File 'lib/rdoc/generator/darkfish.rb', line 766

def generate_class_link(klass, rel_prefix)
  if klass.display?
    %(<code><a href="#{rel_prefix}/#{klass.path}">#{klass.name}</a></code>)
  else
    %(<code>#{klass.name}</code>)
  end
end

#generate_file_files ⇒ Object

Generate a documentation file for each file



376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
# File 'lib/rdoc/generator/darkfish.rb', line 376

def generate_file_files
  page_file     = @template_dir + 'page.rhtml'
  fileinfo_file = @template_dir + 'fileinfo.rhtml'

  # for legacy templates
  filepage_file = @template_dir + 'filepage.rhtml' unless
    page_file.exist? or fileinfo_file.exist?

  return unless
    page_file.exist? or fileinfo_file.exist? or filepage_file.exist?

  debug_msg "Generating file documentation in #{@outputdir}"

  out_file = nil
  current = nil

  @files.each do |file|
    current = file

    next if file.text? && file.full_name == @options.main_page

    if file.text? and page_file.exist?
      generate_page file
      next
    end

    template_file = nil
    out_file = @outputdir + file.path
    debug_msg "  working on %s (%s)" % [file.full_name, out_file]
    rel_prefix = @outputdir.relative_path_from out_file.dirname
    search_index_rel_prefix = rel_prefix
    search_index_rel_prefix += @asset_rel_path if @file_output

    asset_rel_prefix = rel_prefix + @asset_rel_path

    unless filepage_file
      if file.text?
        next unless page_file.exist?
        template_file = page_file
        @title = file.page_name
      else
        next unless fileinfo_file.exist?
        template_file = fileinfo_file
        @title = "File: #{file.base_name}"
      end
    end

    @title += " - #{@options.title}"
    template_file ||= filepage_file

    render_template template_file, out_file do |io|
      here = binding
      # suppress 1.9.3 warning
      here.local_variable_set(:asset_rel_prefix, asset_rel_prefix)
      here.local_variable_set(:current, current)
      here
    end
  end
rescue => e
  error =
    Error.new "error generating #{out_file}: #{e.message} (#{e.class})"
  error.set_backtrace e.backtrace

  raise error
end

#generate_index ⇒ Object

Generate an index page which lists all the classes which are documented.



280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
# File 'lib/rdoc/generator/darkfish.rb', line 280

def generate_index
  template_file = @template_dir + 'index.rhtml'
  return unless template_file.exist?

  debug_msg "Rendering the index page..."

  out_file = @base_dir + @options.op_dir + 'index.html'
  rel_prefix = @outputdir.relative_path_from out_file.dirname
  search_index_rel_prefix = rel_prefix
  search_index_rel_prefix += @asset_rel_path if @file_output

  asset_rel_prefix = rel_prefix + @asset_rel_path

  @title = @options.title
  @main_page = @files.find { |f| f.full_name == @options.main_page }

  render_template template_file, out_file do |io|
    here = binding
    # suppress 1.9.3 warning
    here.local_variable_set(:asset_rel_prefix, asset_rel_prefix)
    # some partials rely on the presence of current variable to render
    here.local_variable_set(:current, @main_page) if @main_page
    here
  end
rescue => e
  error = Error.new \
    "error generating index.html: #{e.message} (#{e.class})"
  error.set_backtrace e.backtrace

  raise error
end

#generate_page(file) ⇒ Object

Generate a page file for file



445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
# File 'lib/rdoc/generator/darkfish.rb', line 445

def generate_page(file)
  template_file = @template_dir + 'page.rhtml'

  out_file = @outputdir + file.path
  debug_msg "  working on %s (%s)" % [file.full_name, out_file]
  rel_prefix = @outputdir.relative_path_from out_file.dirname
  search_index_rel_prefix = rel_prefix
  search_index_rel_prefix += @asset_rel_path if @file_output

  current          = file
  asset_rel_prefix = rel_prefix + @asset_rel_path

  @title = "#{file.page_name} - #{@options.title}"

  debug_msg "  rendering #{out_file}"
  render_template template_file, out_file do |io|
    here = binding
    # suppress 1.9.3 warning
    here.local_variable_set(:current, current)
    here.local_variable_set(:asset_rel_prefix, asset_rel_prefix)
    here
  end
end

#generate_servlet_not_found(message) ⇒ Object

Generates the 404 page for the RDoc servlet



472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
# File 'lib/rdoc/generator/darkfish.rb', line 472

def generate_servlet_not_found(message)
  template_file = @template_dir + 'servlet_not_found.rhtml'
  return unless template_file.exist?

  debug_msg "Rendering the servlet 404 Not Found page..."

  rel_prefix = rel_prefix = ''
  search_index_rel_prefix = rel_prefix
  search_index_rel_prefix += @asset_rel_path if @file_output

  asset_rel_prefix = ''

  @title = 'Not Found'

  render_template template_file do |io|
    here = binding
    # suppress 1.9.3 warning
    here.local_variable_set(:asset_rel_prefix, asset_rel_prefix)
    here
  end
rescue => e
  error = Error.new \
    "error generating servlet_not_found: #{e.message} (#{e.class})"
  error.set_backtrace e.backtrace

  raise error
end

#generate_servlet_root(installed) ⇒ Object

Generates the servlet root page for the RDoc servlet



503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
# File 'lib/rdoc/generator/darkfish.rb', line 503

def generate_servlet_root(installed)
  template_file = @template_dir + 'servlet_root.rhtml'
  return unless template_file.exist?

  debug_msg 'Rendering the servlet root page...'

  rel_prefix = '.'
  asset_rel_prefix = rel_prefix
  search_index_rel_prefix = asset_rel_prefix
  search_index_rel_prefix += @asset_rel_path if @file_output

  @title = 'Local RDoc Documentation'

  render_template template_file do |io| binding end
rescue => e
  error = Error.new \
    "error generating servlet_root: #{e.message} (#{e.class})"
  error.set_backtrace e.backtrace

  raise error
end

#generate_table_of_contents ⇒ Object

Generate an index page which lists all the classes which are documented.



528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
# File 'lib/rdoc/generator/darkfish.rb', line 528

def generate_table_of_contents
  template_file = @template_dir + 'table_of_contents.rhtml'
  return unless template_file.exist?

  debug_msg "Rendering the Table of Contents..."

  out_file = @outputdir + 'table_of_contents.html'
  rel_prefix = @outputdir.relative_path_from out_file.dirname
  search_index_rel_prefix = rel_prefix
  search_index_rel_prefix += @asset_rel_path if @file_output

  asset_rel_prefix = rel_prefix + @asset_rel_path

  @title = "Table of Contents - #{@options.title}"

  render_template template_file, out_file do |io|
    here = binding
    # suppress 1.9.3 warning
    here.local_variable_set(:asset_rel_prefix, asset_rel_prefix)
    here
  end
rescue => e
  error = Error.new \
    "error generating table_of_contents.html: #{e.message} (#{e.class})"
  error.set_backtrace e.backtrace

  raise error
end

#get_sorted_module_list(classes) ⇒ Object

Return a list of the documented modules sorted by salience first, then by name.



271
272
273
274
275
# File 'lib/rdoc/generator/darkfish.rb', line 271

def get_sorted_module_list(classes)
  classes.select do |klass|
    klass.display?
  end.sort
end

#group_classes_by_namespace_for_sidebar(classes) ⇒ Object



799
800
801
802
803
804
805
806
807
808
# File 'lib/rdoc/generator/darkfish.rb', line 799

def group_classes_by_namespace_for_sidebar(classes)
  grouped_classes = classes.group_by do |klass|
    klass.full_name[/\A[^:]++(?:::[^:]++(?=::))*+(?=::[^:]*+\z)/]
  end.select do |_, klasses|
    klasses.any?(&:display?)
  end

  grouped_classes.values.each(&:uniq!)
  grouped_classes
end

#install_rdoc_static_file(source, destination, options) ⇒ Object

:nodoc:



557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
# File 'lib/rdoc/generator/darkfish.rb', line 557

def install_rdoc_static_file(source, destination, options) # :nodoc:
  return unless source.exist?

  begin
    FileUtils.mkdir_p File.dirname(destination), **options

    begin
      FileUtils.ln source, destination, **options
    rescue Errno::EEXIST
      FileUtils.rm destination
      retry
    end
  rescue
    FileUtils.cp source, destination, **options
  end
end

#refresh_store_data ⇒ Object

Refreshes the generator's data from the store. Called by #setup and can be called again after the store has been updated (e.g. in server mode after re-parsing changed files).



592
593
594
595
596
597
# File 'lib/rdoc/generator/darkfish.rb', line 592

def refresh_store_data
  @classes = @store.all_classes_and_modules.sort
  @files   = @store.all_files.sort
  @methods = @classes.flat_map { |m| m.method_list }.sort
  @modsort = get_sorted_module_list @classes
end

#render(file_name) ⇒ Object

Renders the ERb contained in file_name relative to the template directory and returns the result based on the current context.



626
627
628
629
630
631
632
633
634
# File 'lib/rdoc/generator/darkfish.rb', line 626

def render(file_name)
  template_file = @template_dir + file_name

  template = template_for template_file, false, ERBPartial

  template.filename = template_file.to_s

  template.result @context
end

#render_template(template_file, out_file = nil) ⇒ Object

Load and render the erb template in the given template_file and write it out to out_file.

Both template_file and out_file should be Pathname-like objects.

An io will be yielded which must be captured by binding in the caller.



644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
# File 'lib/rdoc/generator/darkfish.rb', line 644

def render_template(template_file, out_file = nil) # :yield: io
  io_output = out_file && !@dry_run && @file_output
  erb_klass = io_output ? ERBIO : ERB

  template = template_for template_file, true, erb_klass

  if io_output
    debug_msg "Outputting to %s" % [out_file.expand_path]

    out_file.dirname.mkpath
    out_file.open 'w', 0644 do |io|
      io.set_encoding @options.encoding

      @context = yield io

      template_result template, @context, template_file
    end
  else
    @context = yield nil

    output = template_result template, @context, template_file

    debug_msg "  would have written %d characters to %s" % [
      output.length, out_file.expand_path
    ] if @dry_run

    output
  end
end

#setup ⇒ Object

Prepares for generation of output from the current directory



577
578
579
580
581
582
583
584
585
# File 'lib/rdoc/generator/darkfish.rb', line 577

def setup
  return if instance_variable_defined? :@outputdir

  @outputdir = Pathname.new(@options.op_dir).expand_path @base_dir

  return unless @store

  refresh_store_data
end

#template_for(file, page = true, klass = ERB) ⇒ Object

Retrieves a cache template for file, if present, or fills the cache.



691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
# File 'lib/rdoc/generator/darkfish.rb', line 691

def template_for(file, page = true, klass = ERB)
  template = @template_cache[file]

  return template if template

  if page
    template = assemble_template file
    erbout = 'io'
  else
    template = file.read(encoding: template_encoding)
    template = template.encode @options.encoding

    file_var = File.basename(file).sub(/\..*/, '')

    erbout = "_erbout_#{file_var}"
  end

  template = klass.new template, trim_mode: '-', eoutvar: erbout
  @template_cache[file] = template
  template
end

#template_result(template, context, template_file) ⇒ Object

Creates the result for template with context. If an error is raised a Pathname template_file will indicate the file where the error occurred.



678
679
680
681
682
683
684
685
686
# File 'lib/rdoc/generator/darkfish.rb', line 678

def template_result(template, context, template_file)
  template.filename = template_file.to_s
  template.result context
rescue NoMethodError => e
  raise Error, "Error while evaluating %s: %s" % [
    template_file.expand_path,
    e.message,
  ], e.backtrace
end

#traverse_classes(klasses, grouped_classes, rel_prefix, solo = false) ⇒ Object



782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
# File 'lib/rdoc/generator/darkfish.rb', line 782

def traverse_classes(klasses, grouped_classes, rel_prefix, solo = false)
  content = +'<ul class="link-list nav-list">'

  klasses.each do |index_klass|
    if children = grouped_classes[index_klass.full_name]
      content << %(<li><details#{solo ? ' open' : ''}><summary>#{generate_class_link(index_klass, rel_prefix)}</summary>)
      content << traverse_classes(children, grouped_classes, rel_prefix)
      content << '</details></li>'
      solo = false
    elsif index_klass.display?
      content << %(<li>#{generate_class_link(index_klass, rel_prefix)}</li>)
    end
  end

  "#{content}</ul>"
end

#write_style_sheet ⇒ Object

Copy over the stylesheet into the appropriate place in the output directory.



190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
# File 'lib/rdoc/generator/darkfish.rb', line 190

def write_style_sheet
  debug_msg "Copying static files"
  options = { :verbose => $DEBUG_RDOC, :noop => @dry_run }

  BUILTIN_STYLE_ITEMS.each do |item|
    install_rdoc_static_file @template_dir + item, "./#{item}", options
  end

  unless @options.template_stylesheets.empty?
    FileUtils.cp @options.template_stylesheets, '.', **options
  end

  Dir[(@template_dir + "{js,images}/**/*").to_s].each do |path|
    next if File.directory? path
    next if File.basename(path) =~ /^\./

    dst = Pathname.new(path).relative_path_from @template_dir

    install_rdoc_static_file @template_dir + path, dst, options
  end
end