Class: RDoc::Markdown::Literals

Inherits:
Object
  • Object
show all
Defined in:
lib/rdoc/markdown/literals.rb

Overview

– This set of literals is for Ruby 1.9 regular expressions and gives full unicode support.

Unlike peg-markdown, this set of literals recognizes Unicode alphanumeric characters, newlines and spaces.

Defined Under Namespace

Classes: KpegPosInfo, MemoEntry, ParseError, RuleInfo

Constant Summary collapse

Rules =
{}

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(str, debug = false) ⇒ Literals

This is distinct from setup_parser so that a standalone parser can redefine #initialize and still have access to the proper parser setup code.



18
19
20
# File 'lib/rdoc/markdown/literals.rb', line 18

def initialize(str, debug=false)
  setup_parser(str, debug)
end

Instance Attribute Details

#failed_ruleObject (readonly)

Returns the value of attribute failed_rule.



209
210
211
# File 'lib/rdoc/markdown/literals.rb', line 209

def failed_rule
  @failed_rule
end

#failing_rule_offsetObject (readonly)

Returns the value of attribute failing_rule_offset.



38
39
40
# File 'lib/rdoc/markdown/literals.rb', line 38

def failing_rule_offset
  @failing_rule_offset
end

#posObject

Returns the value of attribute pos.



39
40
41
# File 'lib/rdoc/markdown/literals.rb', line 39

def pos
  @pos
end

#resultObject

Returns the value of attribute result.



39
40
41
# File 'lib/rdoc/markdown/literals.rb', line 39

def result
  @result
end

#stringObject (readonly)

Returns the value of attribute string.



37
38
39
# File 'lib/rdoc/markdown/literals.rb', line 37

def string
  @string
end

Class Method Details

.rule_info(name, rendered) ⇒ Object



415
416
417
# File 'lib/rdoc/markdown/literals.rb', line 415

def self.rule_info(name, rendered)
  RuleInfo.new(name, rendered)
end

Instance Method Details

#_AlphanumericObject

Alphanumeric = /pWord/



425
426
427
428
429
# File 'lib/rdoc/markdown/literals.rb', line 425

def _Alphanumeric
  _tmp = scan(/\G(?-mix:\p{Word})/)
  set_failed_rule :_Alphanumeric unless _tmp
  return _tmp
end

#_AlphanumericAsciiObject

AlphanumericAscii = /[A-Za-z0-9]/



432
433
434
435
436
# File 'lib/rdoc/markdown/literals.rb', line 432

def _AlphanumericAscii
  _tmp = scan(/\G(?-mix:[A-Za-z0-9])/)
  set_failed_rule :_AlphanumericAscii unless _tmp
  return _tmp
end

#_BOMObject

BOM = “uFEFF”



439
440
441
442
443
# File 'lib/rdoc/markdown/literals.rb', line 439

def _BOM
  _tmp = match_string("uFEFF")
  set_failed_rule :_BOM unless _tmp
  return _tmp
end

#_NewlineObject

Newline = /n|rn?|pZl|pZp/



446
447
448
449
450
# File 'lib/rdoc/markdown/literals.rb', line 446

def _Newline
  _tmp = scan(/\G(?-mix:\n|\r\n?|\p{Zl}|\p{Zp})/)
  set_failed_rule :_Newline unless _tmp
  return _tmp
end

#_NonAlphanumericObject

NonAlphanumeric = /p^Word/



453
454
455
456
457
# File 'lib/rdoc/markdown/literals.rb', line 453

def _NonAlphanumeric
  _tmp = scan(/\G(?-mix:\p{^Word})/)
  set_failed_rule :_NonAlphanumeric unless _tmp
  return _tmp
end

#_SpacecharObject

Spacechar = /t|pZs/



460
461
462
463
464
# File 'lib/rdoc/markdown/literals.rb', line 460

def _Spacechar
  _tmp = scan(/\G(?-mix:\t|\p{Zs})/)
  set_failed_rule :_Spacechar unless _tmp
  return _tmp
end

#apply(rule) ⇒ Object



351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
# File 'lib/rdoc/markdown/literals.rb', line 351

def apply(rule)
  @result = nil
  if m = @memoizations[rule][@pos]
    @pos = m.pos
    if !m.set
      m.left_rec = true
      return nil
    end

    @result = m.result

    return m.ans
  else
    m = MemoEntry.new(nil, @pos)
    @memoizations[rule][@pos] = m
    start_pos = @pos

    ans = __send__ rule

    lr = m.left_rec

    m.move! ans, @pos, @result

    # Don't bother trying to grow the left recursion
    # if it's failing straight away (thus there is no seed)
    if ans and lr
      return grow_lr(rule, nil, start_pos, m)
    else
      return ans
    end
  end
end

#apply_with_args(rule, *args) ⇒ Object



317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
# File 'lib/rdoc/markdown/literals.rb', line 317

def apply_with_args(rule, *args)
  @result = nil
  memo_key = [rule, args]
  if m = @memoizations[memo_key][@pos]
    @pos = m.pos
    if !m.set
      m.left_rec = true
      return nil
    end

    @result = m.result

    return m.ans
  else
    m = MemoEntry.new(nil, @pos)
    @memoizations[memo_key][@pos] = m
    start_pos = @pos

    ans = __send__ rule, *args

    lr = m.left_rec

    m.move! ans, @pos, @result

    # Don't bother trying to grow the left recursion
    # if it's failing straight away (thus there is no seed)
    if ans and lr
      return grow_lr(rule, args, start_pos, m)
    else
      return ans
    end
  end
end

#current_character(target = pos) ⇒ Object



85
86
87
88
89
90
91
92
93
# File 'lib/rdoc/markdown/literals.rb', line 85

def current_character(target=pos)
  if target < 0 || target > string.size
    raise "Target position #{target} is outside of string"
  elsif target == string.size
    ""
  else
    string[target, 1]
  end
end

#current_column(target = pos) ⇒ Object



41
42
43
44
45
46
47
48
49
# File 'lib/rdoc/markdown/literals.rb', line 41

def current_column(target=pos)
  if string[target] == "\n" && (c = string.rindex("\n", target-1) || -1)
    return target - c
  elsif c = string.rindex("\n", target)
    return target - c
  end

  target + 1
end

#current_line(target = pos) ⇒ Object



64
65
66
67
68
69
70
71
72
# File 'lib/rdoc/markdown/literals.rb', line 64

def current_line(target=pos)
  if line = position_line_offsets.bsearch_index {|x| x > target }
    return line + 1
  elsif target == string.size
    past_last = !string.empty? && string[-1]=="\n" ? 1 : 0
    return position_line_offsets.size + past_last
  end
  raise "Target position #{target} is outside of string"
end

#current_pos_info(target = pos) ⇒ Object



97
98
99
100
101
102
103
# File 'lib/rdoc/markdown/literals.rb', line 97

def current_pos_info(target=pos)
  l = current_line target
  c = current_column target
  ln = get_line(l-1)
  chr = string[target,1]
  KpegPosInfo.new(target, l, c, ln, chr)
end

#external_invoke(other, rule, *args) ⇒ Object



298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
# File 'lib/rdoc/markdown/literals.rb', line 298

def external_invoke(other, rule, *args)
  old_pos = @pos
  old_string = @string

  set_string other.string, other.pos

  begin
    if val = __send__(rule, *args)
      other.pos = @pos
      other.result = @result
    else
      other.set_failed_rule "#{self.class}##{rule}"
    end
    val
  ensure
    set_string old_string, old_pos
  end
end

#failure_caretObject



156
157
158
159
# File 'lib/rdoc/markdown/literals.rb', line 156

def failure_caret
  p = current_pos_info @failing_rule_offset
  "#{p.line.chomp}\n#{' ' * (p.col - 1)}^"
end

#failure_characterObject



161
162
163
# File 'lib/rdoc/markdown/literals.rb', line 161

def failure_character
  current_character @failing_rule_offset
end

#failure_infoObject



144
145
146
147
148
149
150
151
152
153
154
# File 'lib/rdoc/markdown/literals.rb', line 144

def failure_info
  l = current_line @failing_rule_offset
  c = current_column @failing_rule_offset

  if @failed_rule.kind_of? Symbol
    info = self.class::Rules[@failed_rule]
    "line #{l}, column #{c}: failed rule '#{info.name}' = '#{info.rendered}'"
  else
    "line #{l}, column #{c}: failed rule '#{@failed_rule}'"
  end
end

#failure_onelineObject



165
166
167
168
169
170
171
172
173
174
# File 'lib/rdoc/markdown/literals.rb', line 165

def failure_oneline
  p = current_pos_info @failing_rule_offset

  if @failed_rule.kind_of? Symbol
    info = self.class::Rules[@failed_rule]
    "@#{p.lno}:#{p.col} failed rule '#{info.name}', got '#{p.char}'"
  else
    "@#{p.lno}:#{p.col} failed rule '#{@failed_rule}', got '#{p.char}'"
  end
end

#get_line(no) ⇒ Object



109
110
111
112
113
114
115
116
117
118
119
# File 'lib/rdoc/markdown/literals.rb', line 109

def get_line(no)
  loff = position_line_offsets
  if no < 0
    raise "Line No is out of range: #{no} < 0"
  elsif no >= loff.size
    raise "Line No is out of range: #{no} >= #{loff.size}"
  end
  lend = loff[no]-1
  lstart = no > 0 ? loff[no-1] : 0
  string[lstart..lend]
end

#get_text(start) ⇒ Object



123
124
125
# File 'lib/rdoc/markdown/literals.rb', line 123

def get_text(start)
  @string[start..@pos-1]
end

#grow_lr(rule, args, start_pos, m) ⇒ Object



384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
# File 'lib/rdoc/markdown/literals.rb', line 384

def grow_lr(rule, args, start_pos, m)
  while true
    @pos = start_pos
    @result = m.result

    if args
      ans = __send__ rule, *args
    else
      ans = __send__ rule
    end
    return nil unless ans

    break if @pos <= m.pos

    m.move! ans, @pos, @result
  end

  @result = m.result
  @pos = m.pos
  return m.ans
end

#linesObject



105
106
107
# File 'lib/rdoc/markdown/literals.rb', line 105

def lines
  string.lines
end

#match_char_range(char_range) ⇒ Object



240
241
242
243
244
245
246
247
248
249
# File 'lib/rdoc/markdown/literals.rb', line 240

def match_char_range(char_range)
  if @pos >= @string_size
    return nil
  elsif !char_range.include?(@string[@pos].ord)
    return nil
  end

  @pos += 1
  true
end

#match_dotObject



211
212
213
214
215
216
217
218
# File 'lib/rdoc/markdown/literals.rb', line 211

def match_dot()
  if @pos >= @string_size
    return nil
  end

  @pos += 1
  true
end

#match_string(str) ⇒ Object



220
221
222
223
224
225
226
227
228
# File 'lib/rdoc/markdown/literals.rb', line 220

def match_string(str)
  len = str.size
  if @string[pos,len] == str
    @pos += len
    return str
  end

  return nil
end

#parse(rule = nil) ⇒ Object



263
264
265
266
267
268
269
270
271
272
273
274
275
# File 'lib/rdoc/markdown/literals.rb', line 263

def parse(rule=nil)
  # We invoke the rules indirectly via apply
  # instead of by just calling them as methods because
  # if the rules use left recursion, apply needs to
  # manage that.

  if !rule
    apply(:_root)
  else
    method = rule.gsub("-","_hyphen_")
    apply :"_#{method}"
  end
end

#position_line_offsetsObject



51
52
53
54
55
56
57
58
59
60
61
# File 'lib/rdoc/markdown/literals.rb', line 51

def position_line_offsets
  unless @position_line_offsets
    @position_line_offsets = []
    total = 0
    string.each_line do |line|
      total += line.size
      @position_line_offsets << total
    end
  end
  @position_line_offsets
end

#raise_errorObject

Raises:



179
180
181
# File 'lib/rdoc/markdown/literals.rb', line 179

def raise_error
  raise ParseError, failure_oneline
end

#scan(reg) ⇒ Object



230
231
232
233
234
235
236
237
# File 'lib/rdoc/markdown/literals.rb', line 230

def scan(reg)
  if m = reg.match(@string, @pos)
    @pos = m.end(0)
    return true
  end

  return nil
end

#set_failed_rule(name) ⇒ Object



202
203
204
205
206
207
# File 'lib/rdoc/markdown/literals.rb', line 202

def set_failed_rule(name)
  if @pos > @failing_rule_offset
    @failed_rule = name
    @failing_rule_offset = @pos
  end
end

#set_string(string, pos) ⇒ Object

Sets the string and current parsing position for the parser.



128
129
130
131
132
133
# File 'lib/rdoc/markdown/literals.rb', line 128

def set_string string, pos
  @string = string
  @string_size = string ? string.size : 0
  @pos = pos
  @position_line_offsets = nil
end

#setup_foreign_grammarObject

:startdoc: :stopdoc:



422
# File 'lib/rdoc/markdown/literals.rb', line 422

def setup_foreign_grammar; end

#setup_parser(str, debug = false) ⇒ Object

Prepares for parsing str. If you define a custom initialize you must call this method before #parse



26
27
28
29
30
31
32
33
34
35
# File 'lib/rdoc/markdown/literals.rb', line 26

def setup_parser(str, debug=false)
  set_string str, 0
  @memoizations = Hash.new { |h,k| h[k] = {} }
  @result = nil
  @failed_rule = nil
  @failing_rule_offset = -1
  @line_offsets = nil

  setup_foreign_grammar
end

#show_error(io = STDOUT) ⇒ Object



183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
# File 'lib/rdoc/markdown/literals.rb', line 183

def show_error(io=STDOUT)
  error_pos = @failing_rule_offset
  p = current_pos_info(error_pos)

  io.puts "On line #{p.lno}, column #{p.col}:"

  if @failed_rule.kind_of? Symbol
    info = self.class::Rules[@failed_rule]
    io.puts "Failed to match '#{info.rendered}' (rule '#{info.name}')"
  else
    io.puts "Failed to match rule '#{@failed_rule}'"
  end

  io.puts "Got: #{p.char.inspect}"
  io.puts "=> #{p.line}"
  io.print(" " * (p.col + 2))
  io.puts "^"
end

#show_posObject



135
136
137
138
139
140
141
142
# File 'lib/rdoc/markdown/literals.rb', line 135

def show_pos
  width = 10
  if @pos < width
    "#{@pos} (\"#{@string[0,@pos]}\" @ \"#{@string[@pos,width]}\")"
  else
    "#{@pos} (\"... #{@string[@pos - width, width]}\" @ \"#{@string[@pos,width]}\")"
  end
end