Class: RDoc::Stats::Normal

Inherits:
Stats::Quiet
  • Object
show all
Defined in:
lib/rdoc/stats/normal.rb

Overview

Stats printer that prints just the files being documented with a progress bar

Instance Method Summary collapse

Instance Method Details

#begin_adding ⇒ Object

:nodoc:



17
18
19
20
# File 'lib/rdoc/stats/normal.rb', line 17

def begin_adding # :nodoc:
  puts "Parsing sources..."
  @last_width = 0
end

#done_adding ⇒ Object

:nodoc:



56
57
58
# File 'lib/rdoc/stats/normal.rb', line 56

def done_adding # :nodoc:
  puts
end

Prints a file with a progress bar



25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
# File 'lib/rdoc/stats/normal.rb', line 25

def print_file(files_so_far, filename)
  progress_bar = sprintf("%3d%% [%2d/%2d]  ",
                         100 * files_so_far / @num_files,
                         files_so_far,
                         @num_files)

  if $stdout.tty?
    # Print a progress bar, but make sure it fits on a single line. Filename
    # will be truncated if necessary.
    size = IO.respond_to?(:console_size) ? IO.console_size : IO.console.winsize
    terminal_width = size[1].to_i.nonzero? || 80
    max_filename_size = (terminal_width - progress_bar.size) - 1

    if filename.size > max_filename_size
      # Turn "some_long_filename.rb" to "...ong_filename.rb"
      filename = filename[(filename.size - max_filename_size) .. -1]
      filename[0..2] = "..."
    end

    # Clean the line with whitespaces so that leftover output from the
    # previous line doesn't show up.
    $stdout.print("\r\e[K") if @last_width && @last_width > 0
    @last_width = progress_bar.size + filename.size
    term = "\r"
  else
    term = "\n"
  end
  $stdout.print(progress_bar, filename, term)
  $stdout.flush
end