Class: RDoc::Task

Inherits:
Rake::TaskLib
  • Object
show all
Defined in:
lib/rdoc/task.rb

Overview

RDoc::Task creates the following rake tasks to generate and clean up RDoc output:

[rdoc] Main task for this RDoc task.

[clobber_rdoc] Delete all the rdoc files. This target is automatically added to the main clobber target.

[rerdoc] Rebuild the rdoc files from scratch, even if they are not out of date.

[rdoc:coverage] Print RDoc coverage report for all rdoc files.

Simple Example:

require 'rdoc/task'

RDoc::Task.new do |rdoc|
rdoc.main = "README.md"
rdoc.rdoc_files.include("README.md", "lib/**/*.rb")
end

The rdoc object passed to the block is an RDoc::Task object. See the attributes list for the RDoc::Task class for available customization options.

Specifying different task names

You may wish to give the task a different name, such as if you are generating two sets of documentation. For instance, if you want to have a development set of documentation including private methods:

require 'rdoc/task'

RDoc::Task.new :rdoc_dev do |rdoc|
rdoc.main = "README.md"
rdoc.rdoc_files.include("README.md", "lib/**/*.rb")
rdoc.options << "--all"
end

The tasks would then be named :rdoc_dev, :clobber_rdoc_dev, and :rerdoc_dev.

If you wish to have completely different task names, then pass a Hash as first argument. With the :rdoc, :clobber_rdoc and :rerdoc options, you can customize the task names to your liking.

For example:

require 'rdoc/task'

RDoc::Task.new(:rdoc => "rdoc", :clobber_rdoc => "rdoc:clean",
             :rerdoc => "rdoc:force")

This will create the tasks :rdoc, :rdoc:clean, :rdoc:force, and :rdoc:coverage.

Direct Known Subclasses

RI::Task

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(name = :rdoc) {|_self| ... } ⇒ Task

Create an RDoc task with the given name. See the RDoc::Task class overview for documentation.

Yields:

  • (_self)

Yield Parameters:

  • _self (RDoc::Task) —

    the object that the method was called on



158
159
160
161
162
163
164
165
166
167
168
# File 'lib/rdoc/task.rb', line 158

def initialize(name = :rdoc) # :yield: self
  defaults

  check_names name

  @name = name

  yield self if block_given?

  define
end

Instance Attribute Details

#external ⇒ Object

Whether to run the rdoc process as an external shell (default is false)



152
153
154
# File 'lib/rdoc/task.rb', line 152

def external
  @external
end

#generator ⇒ Object

Name of format generator (--format) used by rdoc. (defaults to rdoc's default)



137
138
139
# File 'lib/rdoc/task.rb', line 137

def generator
  @generator
end

#main ⇒ Object

Name of file to be used as the main, top level file of the RDoc. (default is none)



126
127
128
# File 'lib/rdoc/task.rb', line 126

def main
  @main
end

#markup ⇒ Object

The markup format; one of: rdoc (the default), markdown, rd, tomdoc. See Markup Formats.



110
111
112
# File 'lib/rdoc/task.rb', line 110

def markup
  @markup
end

#name ⇒ Object

Name of the main, top level task. (default is :rdoc)



105
106
107
# File 'lib/rdoc/task.rb', line 105

def name
  @name
end

#options ⇒ Object

Additional list of options to be passed rdoc. (default is [])



147
148
149
# File 'lib/rdoc/task.rb', line 147

def options
  @options
end

#rdoc_dir ⇒ Object

Name of directory to receive the html output files. (default is "html")



115
116
117
# File 'lib/rdoc/task.rb', line 115

def rdoc_dir
  @rdoc_dir
end

#rdoc_files ⇒ Object

List of files to be included in the rdoc generation. (default is [])



142
143
144
# File 'lib/rdoc/task.rb', line 142

def rdoc_files
  @rdoc_files
end

#template ⇒ Object

Name of template to be used by rdoc. (defaults to rdoc's default)



131
132
133
# File 'lib/rdoc/task.rb', line 131

def template
  @template
end

#title ⇒ Object

Title of RDoc documentation. (defaults to rdoc's default)



120
121
122
# File 'lib/rdoc/task.rb', line 120

def title
  @title
end

Instance Method Details

#before_running_rdoc(&block) ⇒ Object

The block passed to this method will be called just before running the RDoc generator. It is allowed to modify RDoc::Task attributes inside the block.



284
285
286
# File 'lib/rdoc/task.rb', line 284

def before_running_rdoc(&block)
  @before_running_rdoc = block
end

#check_names(names) ⇒ Object

Ensures that names only includes names for the :rdoc, :clobber_rdoc and :rerdoc. If other names are given an ArgumentError is raised.



174
175
176
177
178
179
180
181
182
183
# File 'lib/rdoc/task.rb', line 174

def check_names(names)
  return unless Hash === names

  invalid_options =
    names.keys.map { |k| k.to_sym } - [:rdoc, :clobber_rdoc, :rerdoc]

  unless invalid_options.empty?
    raise ArgumentError, "invalid options: #{invalid_options.join ', '}"
  end
end

#clobber_task_description ⇒ Object

Task description for the clobber rdoc task or its renamed equivalent



188
189
190
# File 'lib/rdoc/task.rb', line 188

def clobber_task_description
  "Remove RDoc HTML files"
end

#coverage_task_description ⇒ Object

Task description for the coverage task or its renamed description



305
306
307
# File 'lib/rdoc/task.rb', line 305

def coverage_task_description
  "Print RDoc coverage report"
end

#defaults ⇒ Object

Sets default task values



195
196
197
198
199
200
201
202
203
204
# File 'lib/rdoc/task.rb', line 195

def defaults
  @name = :rdoc
  @rdoc_files = Rake::FileList.new
  @rdoc_dir = 'html'
  @main = nil
  @title = nil
  @template = nil
  @generator = nil
  @options = []
end

#define ⇒ Object

Create the tasks defined by this task lib.



209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
# File 'lib/rdoc/task.rb', line 209

def define
  desc rdoc_task_description
  task rdoc_task_name

  desc rerdoc_task_description
  task rerdoc_task_name => [clobber_task_name, rdoc_task_name]

  desc clobber_task_description
  task clobber_task_name do
    rm_r @rdoc_dir rescue nil
  end

  task :clobber => [clobber_task_name]

  directory @rdoc_dir

  rdoc_target_deps = [
    @rdoc_files,
    Rake.application.rakefile
  ].flatten.compact

  task rdoc_task_name => [rdoc_target]
  file rdoc_target => rdoc_target_deps do
    @before_running_rdoc.call if @before_running_rdoc
    args = option_list + @rdoc_files

    $stderr.puts "rdoc #{args.join ' '}" if Rake.application.options.trace
    RDoc.new.document args
  end

  namespace rdoc_task_name do
    desc coverage_task_description
    task coverage_task_name do
      @before_running_rdoc.call if @before_running_rdoc
      opts = option_list << "-C"
      args = opts + @rdoc_files

      $stderr.puts "rdoc #{args.join ' '}" if Rake.application.options.trace
      RDoc.new.document args
    rescue SystemExit => exit
      raise "RDoc coverage is incomplete" unless exit.success?
    end

    desc server_task_description
    task "server" do
      @before_running_rdoc.call if @before_running_rdoc
      args = option_list + ["--server"] + @rdoc_files

      $stderr.puts "rdoc #{args.join ' '}" if Rake.application.options.trace
      RDoc.new.document args
    end
  end

  self
end

#option_list ⇒ Object

List of options that will be supplied to RDoc



268
269
270
271
272
273
274
275
276
277
# File 'lib/rdoc/task.rb', line 268

def option_list
  result = @options.dup
  result << "-o"       << @rdoc_dir
  result << "--main"   << main      if main
  result << "--markup" << markup    if markup
  result << "--title"  << title     if title
  result << "-T"       << template  if template
  result << '-f'       << generator if generator
  result
end

#rdoc_task_description ⇒ Object

Task description for the rdoc task or its renamed equivalent



291
292
293
# File 'lib/rdoc/task.rb', line 291

def rdoc_task_description
  'Build RDoc HTML files'
end

#rerdoc_task_description ⇒ Object

Task description for the rerdoc task or its renamed description



298
299
300
# File 'lib/rdoc/task.rb', line 298

def rerdoc_task_description
  "Rebuild RDoc HTML files"
end

#server_task_description ⇒ Object

Task description for the server task



312
313
314
# File 'lib/rdoc/task.rb', line 312

def server_task_description
  "Start a live-reloading documentation server"
end