Class: RDoc::AnyMethod
- Inherits:
-
MethodAttr
- Object
- CodeObject
- MethodAttr
- RDoc::AnyMethod
- Includes:
- TokenStream
- Defined in:
- lib/rdoc/any_method.rb
Overview
AnyMethod is the base class for objects representing methods
Direct Known Subclasses
Constant Summary collapse
- MARSHAL_VERSION =
- 2
-
RDoc 4 Added calls_super Added parent name and class Added section title
- 3
-
RDoc 4.1 Added is_alias_for
3
Constants included from Text
Text::MARKUP_FORMAT, Text::TO_HTML_CHARACTERS
Instance Attribute Summary collapse
-
#c_function ⇒ Object
The C function that implements this method (if it was defined in a C file).
-
#calls_super ⇒ Object
If true this method uses
superto call a superclass version. -
#dont_rename_initialize ⇒ Object
Don't rename #initialize to ::new.
-
#params ⇒ Object
Parameters for this method.
-
#section_title ⇒ Object
The section title of the method (if defined in a C file via
:category:).
Attributes inherited from MethodAttr
#aliases, #block_params, #singleton, #text, #visibility
Attributes inherited from CodeObject
#comment, #document_children, #document_self, #done_documenting, #file, #force_documentation, #line, #metadata, #parent, #received_nodoc, #section, #store, #viewer
Attributes included from Text
Instance Method Summary collapse
-
#add_alias(an_alias, context = nil) ⇒ Object
Adds
an_aliasas an alias for this method incontext. -
#aref_prefix ⇒ Object
Prefix for
arefis 'method'. -
#arglists ⇒ Object
The call_seq or the param_seq with method name, if there is no call_seq.
-
#call_seq ⇒ Object
Different ways to call this method.
-
#call_seq=(call_seq) ⇒ Object
Sets the different ways you can call this method.
-
#initialize(text, name) ⇒ AnyMethod
constructor
Creates a new AnyMethod with a token stream
textandname. -
#is_alias_for ⇒ Object
Loads is_alias_for from the internal name.
-
#marshal_dump ⇒ Object
Dumps this AnyMethod for use by ri.
-
#marshal_load(array) ⇒ Object
Loads this AnyMethod from
array. -
#name ⇒ Object
Method name.
-
#param_list ⇒ Object
A list of this method's method and yield parameters.
-
#param_seq ⇒ Object
Pretty parameter list for this method.
-
#store=(store) ⇒ Object
Sets the store for this method and its referenced code objects.
-
#superclass_method ⇒ Object
For methods that
super, find the superclass method that would be called.
Methods included from TokenStream
#add_token, #add_tokens, #collect_tokens, #pop_token, to_html, #token_stream, #tokens_to_s
Methods inherited from MethodAttr
#<=>, #==, #add_line_numbers, #aref, #documented?, #find_method_or_attribute, #find_see, #full_name, #html_name, #initialize_copy, #initialize_visibility, #inspect, #markup_code, #name_prefix, #output_name, #parent_name, #path, #pretty_name, #pretty_print, #search_record, #see, #to_s, #type
Methods inherited from CodeObject
#display?, #documented?, #each_parent, #file_name, #full_name=, #ignore, #ignored?, #initialize_visibility, #options, #parent_file_name, #parent_name, #record_location, #start_doc, #stop_doc, #suppress, #suppressed?
Methods included from Generator::Markup
#aref_to, #as_href, #cvs_url, #description, #formatter
Methods included from Text
encode_fallback, #expand_tabs, #flush_left, #markup, #normalize_comment, #parse, #snippet, #strip_hashes, #strip_newlines, #strip_stars, #to_html, #wrap
Constructor Details
#initialize(text, name) ⇒ AnyMethod
Creates a new AnyMethod with a token stream text and name
46 47 48 49 50 51 52 53 54 |
# File 'lib/rdoc/any_method.rb', line 46 def initialize text, name super @c_function = nil @dont_rename_initialize = false @token_stream = nil @calls_super = false @superclass_method = nil end |
Instance Attribute Details
#c_function ⇒ Object
The C function that implements this method (if it was defined in a C file)
27 28 29 |
# File 'lib/rdoc/any_method.rb', line 27 def c_function @c_function end |
#calls_super ⇒ Object
If true this method uses super to call a superclass version
39 40 41 |
# File 'lib/rdoc/any_method.rb', line 39 def calls_super @calls_super end |
#dont_rename_initialize ⇒ Object
Don't rename #initialize to ::new
22 23 24 |
# File 'lib/rdoc/any_method.rb', line 22 def dont_rename_initialize @dont_rename_initialize end |
#params ⇒ Object
Parameters for this method
34 35 36 |
# File 'lib/rdoc/any_method.rb', line 34 def params @params end |
#section_title ⇒ Object
The section title of the method (if defined in a C file via :category:)
30 31 32 |
# File 'lib/rdoc/any_method.rb', line 30 def section_title @section_title end |
Instance Method Details
#add_alias(an_alias, context = nil) ⇒ Object
Adds an_alias as an alias for this method in context.
59 60 61 62 63 64 65 66 67 68 69 70 71 |
# File 'lib/rdoc/any_method.rb', line 59 def add_alias an_alias, context = nil method = self.class.new an_alias.text, an_alias.new_name method.record_location an_alias.file method.singleton = self.singleton method.params = self.params method.visibility = self.visibility method.comment = an_alias.comment method.is_alias_for = self @aliases << method context.add_method method if context method end |
#aref_prefix ⇒ Object
Prefix for aref is 'method'.
76 77 78 |
# File 'lib/rdoc/any_method.rb', line 76 def aref_prefix 'method' end |
#arglists ⇒ Object
The call_seq or the param_seq with method name, if there is no call_seq.
Use this for displaying a method's argument lists.
85 86 87 88 89 90 91 |
# File 'lib/rdoc/any_method.rb', line 85 def arglists if @call_seq then @call_seq elsif @params then "#{name}#{param_seq}" end end |
#call_seq ⇒ Object
Different ways to call this method
96 97 98 99 100 101 102 103 104 |
# File 'lib/rdoc/any_method.rb', line 96 def call_seq unless call_seq = _call_seq call_seq = is_alias_for._call_seq if is_alias_for end return unless call_seq deduplicate_call_seq(call_seq) end |
#call_seq=(call_seq) ⇒ Object
Sets the different ways you can call this method. If an empty call_seq is given nil is assumed.
See also #param_seq
112 113 114 115 116 |
# File 'lib/rdoc/any_method.rb', line 112 def call_seq= call_seq return if call_seq.empty? @call_seq = call_seq end |
#is_alias_for ⇒ Object
Loads is_alias_for from the internal name. Returns nil if the alias cannot be found.
122 123 124 125 126 127 128 129 130 131 132 133 134 135 |
# File 'lib/rdoc/any_method.rb', line 122 def is_alias_for # :nodoc: case @is_alias_for when RDoc::MethodAttr then @is_alias_for when Array then return nil unless @store klass_name, singleton, method_name = @is_alias_for return nil unless klass = @store.find_class_or_module(klass_name) @is_alias_for = klass.find_method method_name, singleton end end |
#marshal_dump ⇒ Object
Dumps this AnyMethod for use by ri. See also #marshal_load
140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 |
# File 'lib/rdoc/any_method.rb', line 140 def marshal_dump aliases = @aliases.map do |a| [a.name, parse(a.comment)] end is_alias_for = [ @is_alias_for.parent.full_name, @is_alias_for.singleton, @is_alias_for.name ] if @is_alias_for [ MARSHAL_VERSION, @name, full_name, @singleton, @visibility, parse(@comment), @call_seq, @block_params, aliases, @params, @file.relative_name, @calls_super, @parent.name, @parent.class, @section.title, is_alias_for, ] end |
#marshal_load(array) ⇒ Object
Loads this AnyMethod from array. For a loaded AnyMethod the following methods will return cached values:
-
#full_name
-
#parent_name
177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 |
# File 'lib/rdoc/any_method.rb', line 177 def marshal_load array initialize_visibility @dont_rename_initialize = nil @token_stream = nil @aliases = [] @parent = nil @parent_name = nil @parent_class = nil @section = nil @file = nil version = array[0] @name = array[1] @full_name = array[2] @singleton = array[3] @visibility = array[4] @comment = array[5] @call_seq = array[6] @block_params = array[7] # 8 handled below @params = array[9] # 10 handled below @calls_super = array[11] @parent_name = array[12] @parent_title = array[13] @section_title = array[14] @is_alias_for = array[15] array[8].each do |new_name, comment| add_alias RDoc::Alias.new(nil, @name, new_name, comment, @singleton) end @parent_name ||= if @full_name =~ /#/ then $` else name = @full_name.split('::') name.pop name.join '::' end @file = RDoc::TopLevel.new array[10] if version > 0 end |
#name ⇒ Object
Method name
If the method has no assigned name, it extracts it from #call_seq.
226 227 228 229 230 231 232 233 |
# File 'lib/rdoc/any_method.rb', line 226 def name return @name if @name @name = @call_seq[/^.*?\.(\w+)/, 1] || @call_seq[/^.*?(\w+)/, 1] || @call_seq if @call_seq end |
#param_list ⇒ Object
A list of this method's method and yield parameters. call-seq params are preferred over parsed method and block params.
239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 |
# File 'lib/rdoc/any_method.rb', line 239 def param_list if @call_seq then params = @call_seq.split("\n").last params = params.sub(/.*?\((.*)\)/, '\1') params = params.sub(/(\{|do)\s*\|([^|]*)\|.*/, ',\2') elsif @params then params = @params.sub(/\((.*)\)/, '\1') params << ",#{@block_params}" if @block_params elsif @block_params then params = @block_params else return [] end if @block_params then # If this method has explicit block parameters, remove any explicit # &block params = params.sub(/,?\s*&\w+/, '') else params = params.sub(/\&(\w+)/, '\1') end params = params.gsub(/\s+/, '').split(',').reject(&:empty?) params.map { |param| param.sub(/=.*/, '') } end |
#param_seq ⇒ Object
Pretty parameter list for this method. If the method's parameters were given by call-seq it is preferred over the parsed values.
271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 |
# File 'lib/rdoc/any_method.rb', line 271 def param_seq if @call_seq then params = @call_seq.split("\n").last params = params.sub(/[^( ]+/, '') params = params.sub(/(\|[^|]+\|)\s*\.\.\.\s*(end|\})/, '\1 \2') elsif @params then params = @params.gsub(/\s*\#.*/, '') params = params.tr_s("\n ", " ") params = "(#{params})" unless params[0] == ?( else params = '' end if @block_params then # If this method has explicit block parameters, remove any explicit # &block params = params.sub(/,?\s*&\w+/, '') block = @block_params.tr_s("\n ", " ") if block[0] == ?( block = block.sub(/^\(/, '').sub(/\)/, '') end params << " { |#{block}| ... }" end params end |
#store=(store) ⇒ Object
Sets the store for this method and its referenced code objects.
302 303 304 305 306 |
# File 'lib/rdoc/any_method.rb', line 302 def store= store super @file = @store.add_file @file.full_name if @file end |
#superclass_method ⇒ Object
For methods that super, find the superclass method that would be called.
311 312 313 314 315 316 317 318 319 320 321 322 323 |
# File 'lib/rdoc/any_method.rb', line 311 def superclass_method return unless @calls_super return @superclass_method if @superclass_method parent.each_ancestor do |ancestor| if method = ancestor.method_list.find { |m| m.name == @name } then @superclass_method = method break end end @superclass_method end |