Class: RDoc::Context::Section

Inherits:
Object
  • Object
show all
Includes:
Generator::Markup, Text
Defined in:
lib/rdoc/code_object/context/section.rb,
lib/rdoc/generator/markup.rb

Overview

A section of documentation like:

# :section: The title
# The body

Sections can be referenced multiple times and will be collapsed into a single section.

Constant Summary collapse

MARSHAL_VERSION =

:nodoc:

0

Constants included from Text

Text::MARKUP_FORMAT, Text::SPACE_SEPARATED_LETTER_CLASS

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Generator::Markup

#aref_to, #as_href, #canonical_url, #cvs_url, #formatter

Methods included from Text

decode_legacy_label, expand_tabs, #flush_left, #markup, #normalize_comment, #parse, #snippet, #strip_hashes, #strip_newlines, #strip_stars, to_anchor, #wrap

Constructor Details

#initialize(parent, title, comment, store = nil) ⇒ Section

Creates a new section with title and comment



43
44
45
46
47
48
49
50
51
# File 'lib/rdoc/code_object/context/section.rb', line 43

def initialize(parent, title, comment, store = nil)
  @parent = parent
  @title = title ? title.strip : title
  @store = store

  @comments = []

  add_comment comment
end

Instance Attribute Details

#commentsObject (readonly)

Section comments



23
24
25
# File 'lib/rdoc/code_object/context/section.rb', line 23

def comments
  @comments
end

#parentObject (readonly)

Context this Section lives in



28
29
30
# File 'lib/rdoc/code_object/context/section.rb', line 28

def parent
  @parent
end

#storeObject (readonly)

The RDoc::Store for this object.



38
39
40
# File 'lib/rdoc/code_object/context/section.rb', line 38

def store
  @store
end

#titleObject (readonly)

Section title



33
34
35
# File 'lib/rdoc/code_object/context/section.rb', line 33

def title
  @title
end

Instance Method Details

#==(other) ⇒ Object Also known as: eql?

Sections are equal when they have the same #title



56
57
58
# File 'lib/rdoc/code_object/context/section.rb', line 56

def ==(other)
  self.class === other and @title == other.title
end

#add_comment(comment) ⇒ Object

Adds comment to this section



65
66
67
68
69
70
71
# File 'lib/rdoc/code_object/context/section.rb', line 65

def add_comment(comment)
  Array(comment).each do |c|
    next if c.nil?
    raise TypeError, "unknown comment #{c.inspect}" unless RDoc::Comment === c
    @comments << c unless c.empty?
  end
end

#arefObject

Anchor reference for linking to this section using GitHub-style format.

Examples:

"Section"     -> "section"
"One Two"     -> "one-two"
"[untitled]"  -> "untitled"


81
82
83
84
85
# File 'lib/rdoc/code_object/context/section.rb', line 81

def aref
  title = @title || '[untitled]'

  RDoc::Text.to_anchor(title)
end

#commentObject

Section comment



158
159
160
161
# File 'lib/rdoc/code_object/context/section.rb', line 158

def comment
  return nil if @comments.empty?
  RDoc::Comment.from_document(to_document)
end

#descriptionObject



163
164
165
166
# File 'lib/rdoc/code_object/context/section.rb', line 163

def description
  return '' if @comments.empty?
  markup comment
end

#hashObject

:nodoc:



105
106
107
# File 'lib/rdoc/code_object/context/section.rb', line 105

def hash # :nodoc:
  @title.hash
end

#in_filesObject

The files comments in this section come from



112
113
114
# File 'lib/rdoc/code_object/context/section.rb', line 112

def in_files
  @comments.map(&:file)
end

#inspectObject

:nodoc:



101
102
103
# File 'lib/rdoc/code_object/context/section.rb', line 101

def inspect # :nodoc:
  "#<%s:0x%x %p>" % [self.class, object_id, title]
end

#languageObject



168
169
170
# File 'lib/rdoc/code_object/context/section.rb', line 168

def language
  @comments.first&.language
end

#legacy_arefObject

Legacy anchor reference for backward compatibility.

Examples:

"Section"     -> "section"
"One Two"     -> "one+two"
"[untitled]"  -> "5Buntitled-5D"


95
96
97
98
99
# File 'lib/rdoc/code_object/context/section.rb', line 95

def legacy_aref
  title = @title || '[untitled]'

  CGI.escape(title).gsub('%', '-').sub(/^-/, '')
end

#marshal_dumpObject

Serializes this Section. The title and parsed comment are saved, but not the section parent which must be restored manually.



120
121
122
123
124
125
126
# File 'lib/rdoc/code_object/context/section.rb', line 120

def marshal_dump
  [
    MARSHAL_VERSION,
    @title,
    to_document,
  ]
end

#marshal_load(array) ⇒ Object

De-serializes this Section. The section parent must be restored manually.



131
132
133
134
135
136
# File 'lib/rdoc/code_object/context/section.rb', line 131

def marshal_load(array)
  @parent  = nil

  @title    = array[1]
  @comments = array[2].parts.map { |doc| RDoc::Comment.from_document(doc) }
end

#plain_htmlObject

The section’s title, or ‘Top Section’ if the title is nil.

This is used by the table of contents template so the name is silly.



151
152
153
# File 'lib/rdoc/code_object/context/section.rb', line 151

def plain_html
  @title || 'Top Section'
end

#remove_comment(target_comment) ⇒ Object

Removes a comment from this section if it is from the same file as comment



176
177
178
179
180
# File 'lib/rdoc/code_object/context/section.rb', line 176

def remove_comment(target_comment)
  @comments.delete_if do |stored_comment|
    stored_comment.file == target_comment.file
  end
end

#to_documentObject

Parses comment_location into an RDoc::Markup::Document composed of multiple RDoc::Markup::Documents with their file set.



142
143
144
# File 'lib/rdoc/code_object/context/section.rb', line 142

def to_document
  RDoc::Markup::Document.new(*@comments.map(&:parse))
end