Class: RDoc::Context::Section

Inherits:
Object
  • Object
show all
Includes:
Generator::Markup, Text
Defined in:
lib/rdoc/code_object/context/section.rb,
lib/rdoc/generator/markup.rb

Overview

A section of documentation like:

# :section: The title
# The body

Sections can be referenced multiple times and will be collapsed into a single section.

Constant Summary collapse

MARSHAL_VERSION =

:nodoc:

0

Constants included from Text

Text::MARKUP_FORMAT, Text::SPACE_SEPARATED_LETTER_CLASS, Text::TO_HTML_CHARACTERS

Instance Attribute Summary collapse

Attributes included from Text

#language

Instance Method Summary collapse

Methods included from Generator::Markup

#aref_to, #as_href, #canonical_url, #cvs_url, #description, #formatter

Methods included from Text

encode_fallback, #expand_tabs, #flush_left, #markup, #normalize_comment, #snippet, #strip_hashes, #strip_newlines, #strip_stars, #to_html, #wrap

Constructor Details

#initialize(parent, title, comment) ⇒ Section

Creates a new section with title and comment



42
43
44
45
46
47
48
49
# File 'lib/rdoc/code_object/context/section.rb', line 42

def initialize(parent, title, comment)
  @parent = parent
  @title = title ? title.strip : title

  @comments = []

  add_comment comment
end

Instance Attribute Details

#commentObject (readonly)

Section comment



22
23
24
# File 'lib/rdoc/code_object/context/section.rb', line 22

def comment
  @comment
end

#commentsObject (readonly)

Section comments



27
28
29
# File 'lib/rdoc/code_object/context/section.rb', line 27

def comments
  @comments
end

#parentObject (readonly)

Context this Section lives in



32
33
34
# File 'lib/rdoc/code_object/context/section.rb', line 32

def parent
  @parent
end

#titleObject (readonly)

Section title



37
38
39
# File 'lib/rdoc/code_object/context/section.rb', line 37

def title
  @title
end

Instance Method Details

#==(other) ⇒ Object Also known as: eql?

Sections are equal when they have the same #title



54
55
56
# File 'lib/rdoc/code_object/context/section.rb', line 54

def ==(other)
  self.class === other and @title == other.title
end

#add_comment(comment) ⇒ Object

Adds comment to this section



63
64
65
66
67
68
69
# File 'lib/rdoc/code_object/context/section.rb', line 63

def add_comment(comment)
  comments = Array(comment)
  comments.each do |c|
    extracted_comment = extract_comment(c)
    @comments << extracted_comment unless extracted_comment.empty?
  end
end

#arefObject

Anchor reference for linking to this section



74
75
76
77
78
# File 'lib/rdoc/code_object/context/section.rb', line 74

def aref
  title = @title || '[untitled]'

  CGI.escape(title).gsub('%', '-').sub(/^-/, '')
end

#extract_comment(comment) ⇒ Object

Extracts the comment for this section from the original comment block. If the first line contains :section:, strip it and use the rest. Otherwise remove lines up to the line containing :section:, and look for those lines again at the end and remove them. This lets us write

# :section: The title
# The body


89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'lib/rdoc/code_object/context/section.rb', line 89

def extract_comment(comment)
  case comment
  when nil
    RDoc::Comment.new ''
  when RDoc::Comment then
    if comment.text =~ /^#[ \t]*:section:.*\n/ then
      start = $`
      rest = $'

      comment.text = if start.empty? then
                       rest
                     else
                       rest.sub(/#{start.chomp}\Z/, '')
                     end
    end

    comment
  else
    raise TypeError, "unknown comment #{comment.inspect}"
  end
end

#hashObject

:nodoc:



115
116
117
# File 'lib/rdoc/code_object/context/section.rb', line 115

def hash # :nodoc:
  @title.hash
end

#in_filesObject

The files comments in this section come from



122
123
124
# File 'lib/rdoc/code_object/context/section.rb', line 122

def in_files
  @comments.map(&:file)
end

#inspectObject

:nodoc:



111
112
113
# File 'lib/rdoc/code_object/context/section.rb', line 111

def inspect # :nodoc:
  "#<%s:0x%x %p>" % [self.class, object_id, title]
end

#marshal_dumpObject

Serializes this Section. The title and parsed comment are saved, but not the section parent which must be restored manually.



130
131
132
133
134
135
136
# File 'lib/rdoc/code_object/context/section.rb', line 130

def marshal_dump
  [
    MARSHAL_VERSION,
    @title,
    parse,
  ]
end

#marshal_load(array) ⇒ Object

De-serializes this Section. The section parent must be restored manually.



141
142
143
144
145
146
# File 'lib/rdoc/code_object/context/section.rb', line 141

def marshal_load(array)
  @parent  = nil

  @title    = array[1]
  @comments = array[2].parts.map { |doc| RDoc::Comment.from_document(doc) }
end

#parseObject

Parses comment_location into an RDoc::Markup::Document composed of multiple RDoc::Markup::Documents with their file set.



152
153
154
# File 'lib/rdoc/code_object/context/section.rb', line 152

def parse
  RDoc::Markup::Document.new(*@comments.map(&:parse))
end

#plain_htmlObject

The section’s title, or ‘Top Section’ if the title is nil.

This is used by the table of contents template so the name is silly.



161
162
163
# File 'lib/rdoc/code_object/context/section.rb', line 161

def plain_html
  @title || 'Top Section'
end

#remove_comment(target_comment) ⇒ Object

Removes a comment from this section if it is from the same file as comment



169
170
171
172
173
# File 'lib/rdoc/code_object/context/section.rb', line 169

def remove_comment(target_comment)
  @comments.delete_if do |stored_comment|
    stored_comment.file == target_comment.file
  end
end