Class: RDoc::Markdown::Literals
- Inherits:
-
Object
- Object
- RDoc::Markdown::Literals
show all
- Defined in:
- lib/rdoc/markdown/literals.rb
Overview
– This set of literals is for Ruby 1.9 regular expressions and gives full unicode support.
Unlike peg-markdown, this set of literals recognizes Unicode alphanumeric characters, newlines and spaces.
Defined Under Namespace
Classes: MemoEntry, ParseError, RuleInfo
Constant Summary
collapse
- Rules =
{}
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Constructor Details
#initialize(str, debug = false) ⇒ Literals
This is distinct from setup_parser so that a standalone parser can redefine #initialize and still have access to the proper parser setup code.
18
19
20
|
# File 'lib/rdoc/markdown/literals.rb', line 18
def initialize(str, debug=false)
setup_parser(str, debug)
end
|
Instance Attribute Details
#failed_rule ⇒ Object
Returns the value of attribute failed_rule.
180
181
182
|
# File 'lib/rdoc/markdown/literals.rb', line 180
def failed_rule
@failed_rule
end
|
#failing_rule_offset ⇒ Object
Returns the value of attribute failing_rule_offset.
38
39
40
|
# File 'lib/rdoc/markdown/literals.rb', line 38
def failing_rule_offset
@failing_rule_offset
end
|
#pos ⇒ Object
Returns the value of attribute pos.
39
40
41
|
# File 'lib/rdoc/markdown/literals.rb', line 39
def pos
@pos
end
|
#result ⇒ Object
Returns the value of attribute result.
39
40
41
|
# File 'lib/rdoc/markdown/literals.rb', line 39
def result
@result
end
|
#string ⇒ Object
Returns the value of attribute string.
37
38
39
|
# File 'lib/rdoc/markdown/literals.rb', line 37
def string
@string
end
|
Class Method Details
.rule_info(name, rendered) ⇒ Object
373
374
375
|
# File 'lib/rdoc/markdown/literals.rb', line 373
def self.rule_info(name, rendered)
RuleInfo.new(name, rendered)
end
|
Instance Method Details
#_Alphanumeric ⇒ Object
383
384
385
386
387
|
# File 'lib/rdoc/markdown/literals.rb', line 383
def _Alphanumeric
_tmp = scan(/\G(?-mix:\p{Word})/)
set_failed_rule :_Alphanumeric unless _tmp
return _tmp
end
|
#_AlphanumericAscii ⇒ Object
AlphanumericAscii = /[A-Za-z0-9]/
390
391
392
393
394
|
# File 'lib/rdoc/markdown/literals.rb', line 390
def _AlphanumericAscii
_tmp = scan(/\G(?-mix:[A-Za-z0-9])/)
set_failed_rule :_AlphanumericAscii unless _tmp
return _tmp
end
|
#_BOM ⇒ Object
397
398
399
400
401
|
# File 'lib/rdoc/markdown/literals.rb', line 397
def _BOM
_tmp = match_string("uFEFF")
set_failed_rule :_BOM unless _tmp
return _tmp
end
|
#_Newline ⇒ Object
Newline = /n|rn?|pZl|pZp/
404
405
406
407
408
|
# File 'lib/rdoc/markdown/literals.rb', line 404
def _Newline
_tmp = scan(/\G(?-mix:\n|\r\n?|\p{Zl}|\p{Zp})/)
set_failed_rule :_Newline unless _tmp
return _tmp
end
|
#_NonAlphanumeric ⇒ Object
NonAlphanumeric = /p^Word/
411
412
413
414
415
|
# File 'lib/rdoc/markdown/literals.rb', line 411
def _NonAlphanumeric
_tmp = scan(/\G(?-mix:\p{^Word})/)
set_failed_rule :_NonAlphanumeric unless _tmp
return _tmp
end
|
#_Spacechar ⇒ Object
418
419
420
421
422
|
# File 'lib/rdoc/markdown/literals.rb', line 418
def _Spacechar
_tmp = scan(/\G(?-mix:\t|\p{Zs})/)
set_failed_rule :_Spacechar unless _tmp
return _tmp
end
|
#apply(rule) ⇒ Object
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
|
# File 'lib/rdoc/markdown/literals.rb', line 310
def apply(rule)
if m = @memoizations[rule][@pos]
@pos = m.pos
if !m.set
m.left_rec = true
return nil
end
@result = m.result
return m.ans
else
m = MemoEntry.new(nil, @pos)
@memoizations[rule][@pos] = m
start_pos = @pos
ans = __send__ rule
lr = m.left_rec
m.move! ans, @pos, @result
if ans and lr
return grow_lr(rule, nil, start_pos, m)
else
return ans
end
end
end
|
#apply_with_args(rule, *args) ⇒ Object
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
|
# File 'lib/rdoc/markdown/literals.rb', line 277
def apply_with_args(rule, *args)
memo_key = [rule, args]
if m = @memoizations[memo_key][@pos]
@pos = m.pos
if !m.set
m.left_rec = true
return nil
end
@result = m.result
return m.ans
else
m = MemoEntry.new(nil, @pos)
@memoizations[memo_key][@pos] = m
start_pos = @pos
ans = __send__ rule, *args
lr = m.left_rec
m.move! ans, @pos, @result
if ans and lr
return grow_lr(rule, args, start_pos, m)
else
return ans
end
end
end
|
#current_column(target = pos) ⇒ Object
41
42
43
44
45
46
47
|
# File 'lib/rdoc/markdown/literals.rb', line 41
def current_column(target=pos)
if c = string.rindex("\n", target-1)
return target - c - 1
end
target + 1
end
|
#current_line(target = pos) ⇒ Object
50
51
52
53
54
55
56
57
58
59
60
61
|
# File 'lib/rdoc/markdown/literals.rb', line 50
def current_line(target=pos)
unless @line_offsets
@line_offsets = []
total = 0
string.each_line do |line|
total += line.size
@line_offsets << total
end
end
@line_offsets.bsearch_index {|x| x >= target } + 1 || -1
end
|
#external_invoke(other, rule, *args) ⇒ Object
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
|
# File 'lib/rdoc/markdown/literals.rb', line 258
def external_invoke(other, rule, *args)
old_pos = @pos
old_string = @string
set_string other.string, other.pos
begin
if val = __send__(rule, *args)
other.pos = @pos
other.result = @result
else
other.set_failed_rule "#{self.class}##{rule}"
end
val
ensure
set_string old_string, old_pos
end
end
|
#failure_caret ⇒ Object
117
118
119
120
121
122
123
|
# File 'lib/rdoc/markdown/literals.rb', line 117
def failure_caret
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
line = lines[l-1]
"#{line}\n#{' ' * (c - 1)}^"
end
|
#failure_character ⇒ Object
125
126
127
128
129
|
# File 'lib/rdoc/markdown/literals.rb', line 125
def failure_character
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
lines[l-1][c-1, 1]
end
|
#failure_info ⇒ Object
105
106
107
108
109
110
111
112
113
114
115
|
# File 'lib/rdoc/markdown/literals.rb', line 105
def failure_info
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
"line #{l}, column #{c}: failed rule '#{info.name}' = '#{info.rendered}'"
else
"line #{l}, column #{c}: failed rule '#{@failed_rule}'"
end
end
|
#failure_oneline ⇒ Object
131
132
133
134
135
136
137
138
139
140
141
142
143
|
# File 'lib/rdoc/markdown/literals.rb', line 131
def failure_oneline
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
char = lines[l-1][c-1, 1]
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
"@#{l}:#{c} failed rule '#{info.name}', got '#{char}'"
else
"@#{l}:#{c} failed rule '#{@failed_rule}', got '#{char}'"
end
end
|
#get_byte ⇒ Object
202
203
204
205
206
207
208
209
210
|
# File 'lib/rdoc/markdown/literals.rb', line 202
def get_byte
if @pos >= @string_size
return nil
end
s = @string[@pos].ord
@pos += 1
s
end
|
#get_text(start) ⇒ Object
85
86
87
|
# File 'lib/rdoc/markdown/literals.rb', line 85
def get_text(start)
@string[start..@pos-1]
end
|
#grow_lr(rule, args, start_pos, m) ⇒ Object
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
|
# File 'lib/rdoc/markdown/literals.rb', line 342
def grow_lr(rule, args, start_pos, m)
while true
@pos = start_pos
@result = m.result
if args
ans = __send__ rule, *args
else
ans = __send__ rule
end
return nil unless ans
break if @pos <= m.pos
m.move! ans, @pos, @result
end
@result = m.result
@pos = m.pos
return m.ans
end
|
#lines ⇒ Object
77
78
79
80
81
|
# File 'lib/rdoc/markdown/literals.rb', line 77
def lines
lines = []
string.each_line { |l| lines << l }
lines
end
|
#match_string(str) ⇒ Object
182
183
184
185
186
187
188
189
190
|
# File 'lib/rdoc/markdown/literals.rb', line 182
def match_string(str)
len = str.size
if @string[pos,len] == str
@pos += len
return str
end
return nil
end
|
#parse(rule = nil) ⇒ Object
223
224
225
226
227
228
229
230
231
232
233
234
235
|
# File 'lib/rdoc/markdown/literals.rb', line 223
def parse(rule=nil)
if !rule
apply(:_root)
else
method = rule.gsub("-","_hyphen_")
apply :"_#{method}"
end
end
|
#raise_error ⇒ Object
148
149
150
|
# File 'lib/rdoc/markdown/literals.rb', line 148
def raise_error
raise ParseError, failure_oneline
end
|
#scan(reg) ⇒ Object
192
193
194
195
196
197
198
199
|
# File 'lib/rdoc/markdown/literals.rb', line 192
def scan(reg)
if m = reg.match(@string, @pos)
@pos = m.end(0)
return true
end
return nil
end
|
#set_failed_rule(name) ⇒ Object
173
174
175
176
177
178
|
# File 'lib/rdoc/markdown/literals.rb', line 173
def set_failed_rule(name)
if @pos > @failing_rule_offset
@failed_rule = name
@failing_rule_offset = @pos
end
end
|
#set_string(string, pos) ⇒ Object
Sets the string and current parsing position for the parser.
90
91
92
93
94
|
# File 'lib/rdoc/markdown/literals.rb', line 90
def set_string string, pos
@string = string
@string_size = string ? string.size : 0
@pos = pos
end
|
#setup_foreign_grammar ⇒ Object
380
|
# File 'lib/rdoc/markdown/literals.rb', line 380
def setup_foreign_grammar; end
|
#setup_parser(str, debug = false) ⇒ Object
Prepares for parsing str. If you define a custom initialize you must call this method before #parse
26
27
28
29
30
31
32
33
34
35
|
# File 'lib/rdoc/markdown/literals.rb', line 26
def setup_parser(str, debug=false)
set_string str, 0
@memoizations = Hash.new { |h,k| h[k] = {} }
@result = nil
@failed_rule = nil
@failing_rule_offset = -1
@line_offsets = nil
setup_foreign_grammar
end
|
#show_error(io = STDOUT) ⇒ Object
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
|
# File 'lib/rdoc/markdown/literals.rb', line 152
def show_error(io=STDOUT)
error_pos = @failing_rule_offset
line_no = current_line(error_pos)
col_no = current_column(error_pos)
io.puts "On line #{line_no}, column #{col_no}:"
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
io.puts "Failed to match '#{info.rendered}' (rule '#{info.name}')"
else
io.puts "Failed to match rule '#{@failed_rule}'"
end
io.puts "Got: #{string[error_pos,1].inspect}"
line = lines[line_no-1]
io.puts "=> #{line}"
io.print(" " * (col_no + 3))
io.puts "^"
end
|
#show_pos ⇒ Object
96
97
98
99
100
101
102
103
|
# File 'lib/rdoc/markdown/literals.rb', line 96
def show_pos
width = 10
if @pos < width
"#{@pos} (\"#{@string[0,@pos]}\" @ \"#{@string[@pos,width]}\")"
else
"#{@pos} (\"... #{@string[@pos - width, width]}\" @ \"#{@string[@pos,width]}\")"
end
end
|