Class: RDoc::Markdown::Literals
- Inherits:
-
Object
- Object
- RDoc::Markdown::Literals
show all
- Defined in:
- lib/rdoc/markdown/literals.rb
Overview
– This set of literals is for Ruby 1.9 regular expressions and gives full unicode support.
Unlike peg-markdown, this set of literals recognizes Unicode alphanumeric characters, newlines and spaces.
Defined Under Namespace
Classes: KpegPosInfo, MemoEntry, ParseError, RuleInfo
Constant Summary
collapse
- Rules =
{}
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Constructor Details
#initialize(str, debug = false) ⇒ Literals
This is distinct from setup_parser so that a standalone parser can redefine #initialize and still have access to the proper parser setup code.
18
19
20
|
# File 'lib/rdoc/markdown/literals.rb', line 18
def initialize(str, debug=false)
setup_parser(str, debug)
end
|
Instance Attribute Details
#failed_rule ⇒ Object
Returns the value of attribute failed_rule.
201
202
203
|
# File 'lib/rdoc/markdown/literals.rb', line 201
def failed_rule
@failed_rule
end
|
#failing_rule_offset ⇒ Object
Returns the value of attribute failing_rule_offset.
38
39
40
|
# File 'lib/rdoc/markdown/literals.rb', line 38
def failing_rule_offset
@failing_rule_offset
end
|
#pos ⇒ Object
Returns the value of attribute pos.
39
40
41
|
# File 'lib/rdoc/markdown/literals.rb', line 39
def pos
@pos
end
|
#result ⇒ Object
Returns the value of attribute result.
39
40
41
|
# File 'lib/rdoc/markdown/literals.rb', line 39
def result
@result
end
|
#string ⇒ Object
Returns the value of attribute string.
37
38
39
|
# File 'lib/rdoc/markdown/literals.rb', line 37
def string
@string
end
|
Class Method Details
.rule_info(name, rendered) ⇒ Object
396
397
398
|
# File 'lib/rdoc/markdown/literals.rb', line 396
def self.rule_info(name, rendered)
RuleInfo.new(name, rendered)
end
|
Instance Method Details
#_Alphanumeric ⇒ Object
406
407
408
409
410
|
# File 'lib/rdoc/markdown/literals.rb', line 406
def _Alphanumeric
_tmp = scan(/\G(?-mix:\p{Word})/)
set_failed_rule :_Alphanumeric unless _tmp
return _tmp
end
|
#_AlphanumericAscii ⇒ Object
AlphanumericAscii = /[A-Za-z0-9]/
413
414
415
416
417
|
# File 'lib/rdoc/markdown/literals.rb', line 413
def _AlphanumericAscii
_tmp = scan(/\G(?-mix:[A-Za-z0-9])/)
set_failed_rule :_AlphanumericAscii unless _tmp
return _tmp
end
|
#_BOM ⇒ Object
420
421
422
423
424
|
# File 'lib/rdoc/markdown/literals.rb', line 420
def _BOM
_tmp = match_string("uFEFF")
set_failed_rule :_BOM unless _tmp
return _tmp
end
|
#_Newline ⇒ Object
Newline = /n|rn?|pZl|pZp/
427
428
429
430
431
|
# File 'lib/rdoc/markdown/literals.rb', line 427
def _Newline
_tmp = scan(/\G(?-mix:\n|\r\n?|\p{Zl}|\p{Zp})/)
set_failed_rule :_Newline unless _tmp
return _tmp
end
|
#_NonAlphanumeric ⇒ Object
NonAlphanumeric = /p^Word/
434
435
436
437
438
|
# File 'lib/rdoc/markdown/literals.rb', line 434
def _NonAlphanumeric
_tmp = scan(/\G(?-mix:\p{^Word})/)
set_failed_rule :_NonAlphanumeric unless _tmp
return _tmp
end
|
#_Spacechar ⇒ Object
441
442
443
444
445
|
# File 'lib/rdoc/markdown/literals.rb', line 441
def _Spacechar
_tmp = scan(/\G(?-mix:\t|\p{Zs})/)
set_failed_rule :_Spacechar unless _tmp
return _tmp
end
|
#apply(rule) ⇒ Object
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
|
# File 'lib/rdoc/markdown/literals.rb', line 332
def apply(rule)
@result = nil
if m = @memoizations[rule][@pos]
@pos = m.pos
if !m.set
m.left_rec = true
return nil
end
@result = m.result
return m.ans
else
m = MemoEntry.new(nil, @pos)
@memoizations[rule][@pos] = m
start_pos = @pos
ans = __send__ rule
lr = m.left_rec
m.move! ans, @pos, @result
if ans and lr
return grow_lr(rule, nil, start_pos, m)
else
return ans
end
end
end
|
#apply_with_args(rule, *args) ⇒ Object
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
|
# File 'lib/rdoc/markdown/literals.rb', line 298
def apply_with_args(rule, *args)
@result = nil
memo_key = [rule, args]
if m = @memoizations[memo_key][@pos]
@pos = m.pos
if !m.set
m.left_rec = true
return nil
end
@result = m.result
return m.ans
else
m = MemoEntry.new(nil, @pos)
@memoizations[memo_key][@pos] = m
start_pos = @pos
ans = __send__ rule, *args
lr = m.left_rec
m.move! ans, @pos, @result
if ans and lr
return grow_lr(rule, args, start_pos, m)
else
return ans
end
end
end
|
#current_character(target = pos) ⇒ Object
80
81
82
83
84
85
|
# File 'lib/rdoc/markdown/literals.rb', line 80
def current_character(target=pos)
if target < 0 || target >= string.size
raise "Target position #{target} is outside of string"
end
string[target, 1]
end
|
#current_column(target = pos) ⇒ Object
41
42
43
44
45
46
47
48
49
|
# File 'lib/rdoc/markdown/literals.rb', line 41
def current_column(target=pos)
if string[target] == "\n" && (c = string.rindex("\n", target-1) || -1)
return target - c
elsif c = string.rindex("\n", target)
return target - c
end
target + 1
end
|
#current_line(target = pos) ⇒ Object
64
65
66
67
68
69
|
# File 'lib/rdoc/markdown/literals.rb', line 64
def current_line(target=pos)
if line = position_line_offsets.bsearch_index {|x| x > target }
return line + 1
end
raise "Target position #{target} is outside of string"
end
|
#current_pos_info(target = pos) ⇒ Object
89
90
91
92
93
94
95
|
# File 'lib/rdoc/markdown/literals.rb', line 89
def current_pos_info(target=pos)
l = current_line target
c = current_column target
ln = get_line(l-1)
chr = string[target,1]
KpegPosInfo.new(target, l, c, ln, chr)
end
|
#external_invoke(other, rule, *args) ⇒ Object
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
|
# File 'lib/rdoc/markdown/literals.rb', line 279
def external_invoke(other, rule, *args)
old_pos = @pos
old_string = @string
set_string other.string, other.pos
begin
if val = __send__(rule, *args)
other.pos = @pos
other.result = @result
else
other.set_failed_rule "#{self.class}##{rule}"
end
val
ensure
set_string old_string, old_pos
end
end
|
#failure_caret ⇒ Object
148
149
150
151
|
# File 'lib/rdoc/markdown/literals.rb', line 148
def failure_caret
p = current_pos_info @failing_rule_offset
"#{p.line.chomp}\n#{' ' * (p.col - 1)}^"
end
|
#failure_character ⇒ Object
153
154
155
|
# File 'lib/rdoc/markdown/literals.rb', line 153
def failure_character
current_character @failing_rule_offset
end
|
#failure_info ⇒ Object
136
137
138
139
140
141
142
143
144
145
146
|
# File 'lib/rdoc/markdown/literals.rb', line 136
def failure_info
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
"line #{l}, column #{c}: failed rule '#{info.name}' = '#{info.rendered}'"
else
"line #{l}, column #{c}: failed rule '#{@failed_rule}'"
end
end
|
#failure_oneline ⇒ Object
157
158
159
160
161
162
163
164
165
166
|
# File 'lib/rdoc/markdown/literals.rb', line 157
def failure_oneline
p = current_pos_info @failing_rule_offset
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
"@#{p.lno}:#{p.col} failed rule '#{info.name}', got '#{p.char}'"
else
"@#{p.lno}:#{p.col} failed rule '#{@failed_rule}', got '#{p.char}'"
end
end
|
#get_byte ⇒ Object
223
224
225
226
227
228
229
230
231
|
# File 'lib/rdoc/markdown/literals.rb', line 223
def get_byte
if @pos >= @string_size
return nil
end
s = @string[@pos].ord
@pos += 1
s
end
|
#get_line(no) ⇒ Object
101
102
103
104
105
106
107
108
109
110
111
|
# File 'lib/rdoc/markdown/literals.rb', line 101
def get_line(no)
loff = position_line_offsets
if no < 0
raise "Line No is out of range: #{no} < 0"
elsif no >= loff.size
raise "Line No is out of range: #{no} >= #{loff.size}"
end
lend = loff[no]-1
lstart = no > 0 ? loff[no-1] : 0
string[lstart..lend]
end
|
#get_text(start) ⇒ Object
115
116
117
|
# File 'lib/rdoc/markdown/literals.rb', line 115
def get_text(start)
@string[start..@pos-1]
end
|
#grow_lr(rule, args, start_pos, m) ⇒ Object
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
|
# File 'lib/rdoc/markdown/literals.rb', line 365
def grow_lr(rule, args, start_pos, m)
while true
@pos = start_pos
@result = m.result
if args
ans = __send__ rule, *args
else
ans = __send__ rule
end
return nil unless ans
break if @pos <= m.pos
m.move! ans, @pos, @result
end
@result = m.result
@pos = m.pos
return m.ans
end
|
#lines ⇒ Object
97
98
99
|
# File 'lib/rdoc/markdown/literals.rb', line 97
def lines
string.lines
end
|
#match_string(str) ⇒ Object
203
204
205
206
207
208
209
210
211
|
# File 'lib/rdoc/markdown/literals.rb', line 203
def match_string(str)
len = str.size
if @string[pos,len] == str
@pos += len
return str
end
return nil
end
|
#parse(rule = nil) ⇒ Object
244
245
246
247
248
249
250
251
252
253
254
255
256
|
# File 'lib/rdoc/markdown/literals.rb', line 244
def parse(rule=nil)
if !rule
apply(:_root)
else
method = rule.gsub("-","_hyphen_")
apply :"_#{method}"
end
end
|
#position_line_offsets ⇒ Object
51
52
53
54
55
56
57
58
59
60
61
|
# File 'lib/rdoc/markdown/literals.rb', line 51
def position_line_offsets
unless @position_line_offsets
@position_line_offsets = []
total = 0
string.each_line do |line|
total += line.size
@position_line_offsets << total
end
end
@position_line_offsets
end
|
#raise_error ⇒ Object
171
172
173
|
# File 'lib/rdoc/markdown/literals.rb', line 171
def raise_error
raise ParseError, failure_oneline
end
|
#scan(reg) ⇒ Object
213
214
215
216
217
218
219
220
|
# File 'lib/rdoc/markdown/literals.rb', line 213
def scan(reg)
if m = reg.match(@string, @pos)
@pos = m.end(0)
return true
end
return nil
end
|
#set_failed_rule(name) ⇒ Object
194
195
196
197
198
199
|
# File 'lib/rdoc/markdown/literals.rb', line 194
def set_failed_rule(name)
if @pos > @failing_rule_offset
@failed_rule = name
@failing_rule_offset = @pos
end
end
|
#set_string(string, pos) ⇒ Object
Sets the string and current parsing position for the parser.
120
121
122
123
124
125
|
# File 'lib/rdoc/markdown/literals.rb', line 120
def set_string string, pos
@string = string
@string_size = string ? string.size : 0
@pos = pos
@position_line_offsets = nil
end
|
#setup_foreign_grammar ⇒ Object
403
|
# File 'lib/rdoc/markdown/literals.rb', line 403
def setup_foreign_grammar; end
|
#setup_parser(str, debug = false) ⇒ Object
Prepares for parsing str. If you define a custom initialize you must call this method before #parse
26
27
28
29
30
31
32
33
34
35
|
# File 'lib/rdoc/markdown/literals.rb', line 26
def setup_parser(str, debug=false)
set_string str, 0
@memoizations = Hash.new { |h,k| h[k] = {} }
@result = nil
@failed_rule = nil
@failing_rule_offset = -1
@line_offsets = nil
setup_foreign_grammar
end
|
#show_error(io = STDOUT) ⇒ Object
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
|
# File 'lib/rdoc/markdown/literals.rb', line 175
def show_error(io=STDOUT)
error_pos = @failing_rule_offset
p = current_pos_info(error_pos)
io.puts "On line #{p.lno}, column #{p.col}:"
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
io.puts "Failed to match '#{info.rendered}' (rule '#{info.name}')"
else
io.puts "Failed to match rule '#{@failed_rule}'"
end
io.puts "Got: #{p.char.inspect}"
io.puts "=> #{p.line}"
io.print(" " * (p.col + 2))
io.puts "^"
end
|
#show_pos ⇒ Object
127
128
129
130
131
132
133
134
|
# File 'lib/rdoc/markdown/literals.rb', line 127
def show_pos
width = 10
if @pos < width
"#{@pos} (\"#{@string[0,@pos]}\" @ \"#{@string[@pos,width]}\")"
else
"#{@pos} (\"... #{@string[@pos - width, width]}\" @ \"#{@string[@pos,width]}\")"
end
end
|