Class: RDoc::Store
- Inherits:
-
Object
- Object
- RDoc::Store
- Defined in:
- lib/rdoc/store.rb
Overview
A set of rdoc data for a single project (gem, path, etc.).
The store manages reading and writing ri data for a project and maintains a cache of methods, classes and ancestors in the store.
The store maintains a #cache of its contents for faster lookup. After adding items to the store it must be flushed using #save_cache. The cache contains the following structures:
@cache = {
:ancestors => {}, # class name => ancestor names
:attributes => {}, # class name => attributes
:class_methods => {}, # class name => class methods
:instance_methods => {}, # class name => instance methods
:modules => [], # classes and modules in this store
:pages => [], # page names
}
– TODO need to prune classes
Defined Under Namespace
Classes: Error, MissingFileError
Instance Attribute Summary collapse
-
#c_class_variables ⇒ Object
readonly
Maps C variables to class or module names for each parsed C file.
-
#c_enclosure_classes ⇒ Object
readonly
Stores the name of the C variable a class belongs to.
-
#c_enclosure_names ⇒ Object
readonly
:nodoc:.
-
#c_singleton_class_variables ⇒ Object
readonly
Maps C variables to singleton class names for each parsed C file.
-
#cache ⇒ Object
readonly
The contents of the Store.
-
#dry_run ⇒ Object
If true this Store will not write any files.
-
#encoding ⇒ Object
The encoding of the contents in the Store.
-
#options ⇒ Object
readonly
Returns the value of attribute options.
-
#path ⇒ Object
Path this store reads or writes.
-
#type ⇒ Object
Type of ri datastore this was loaded from.
-
#unmatched_constant_alias ⇒ Object
readonly
The lazy constants alias will be discovered in passing.
Instance Method Summary collapse
-
#add_c_enclosure(variable, namespace) ⇒ Object
Adds
moduleas an enclosure (namespace) for the givenvariablefor C files. -
#add_c_variables(c_parser) ⇒ Object
Adds C variables from an RDoc::Parser::C.
-
#add_file(absolute_name, relative_name: absolute_name, parser: nil) ⇒ Object
Adds the file with
nameas an RDoc::TopLevel to the store. -
#all_classes ⇒ Object
Returns all classes discovered by RDoc.
-
#all_classes_and_modules ⇒ Object
Returns all classes and modules discovered by RDoc.
-
#all_files ⇒ Object
All TopLevels known to RDoc.
-
#all_modules ⇒ Object
Returns all modules discovered by RDoc.
-
#ancestors ⇒ Object
Ancestors cache accessor.
-
#attributes ⇒ Object
Attributes cache accessor.
-
#cache_path ⇒ Object
Path to the cache file.
-
#cache_text_file(relative_name) ⇒ Object
Caches
relative_namein the text files hash, if it is a text file. -
#class_file(klass_name) ⇒ Object
Path to the ri data for
klass_name. -
#class_methods ⇒ Object
Class methods cache accessor.
-
#class_path(klass_name) ⇒ Object
Path where data for
klass_namewill be stored (methods or class data). -
#classes_hash ⇒ Object
Hash of all classes known to RDoc.
-
#clean_cache_collection(collection) ⇒ Object
Removes empty items and ensures item in each collection are unique and sorted.
-
#cleanup_stale_contributions ⇒ Object
Removes stale empty placeholders left by
clear_file_contributionswithkeep_position: true. -
#clear_file_contributions(relative_name, keep_position: false) ⇒ Object
Removes a file’s contributions (methods, constants, comments, etc.) from its classes and modules.
-
#clear_rbs_signatures ⇒ Object
:nodoc:.
-
#complete(min_visibility) ⇒ Object
Prepares the RDoc code object tree for use by a generator.
-
#files_hash ⇒ Object
Hash of all files known to RDoc.
-
#find_c_enclosure(variable) ⇒ Object
Finds the enclosure (namespace) for the given C
variable. -
#find_class_named(name) ⇒ Object
Finds the class with
namein all discovered classes. -
#find_class_named_from(name, from) ⇒ Object
Finds the class with
namestarting in namespacefrom. -
#find_class_or_module(name) ⇒ Object
Finds the class or module with
name. -
#find_file_named(name) ⇒ Object
Finds the file with
namein all discovered files. -
#find_module_named(name) ⇒ Object
Finds the module with
namein all discovered modules. -
#find_text_page(file_name) ⇒ Object
Returns the RDoc::TopLevel that is a text file and has the given
file_name. -
#find_unique(all_hash) ⇒ Object
Finds unique classes/modules defined in
all_hash, and returns them as an array. -
#fix_basic_object_inheritance ⇒ Object
Fixes the erroneous
BasicObject < Objectin 1.9. -
#friendly_path ⇒ Object
Friendly rendition of #path.
-
#initialize(options, path: nil, type: nil) ⇒ Store
constructor
Creates a new Store of
typethat will load or save topath. -
#inspect ⇒ Object
:nodoc:.
-
#instance_methods ⇒ Object
Instance methods cache accessor.
-
#invalidate_type_name_lookup ⇒ Object
Invalidates the cached type name lookup.
-
#load_all ⇒ Object
Loads all items from this store into memory.
-
#load_cache ⇒ Object
Loads cache file for this store.
-
#load_class(klass_name) ⇒ Object
Loads ri data for
klass_nameand hooks it up to this store. -
#load_class_data(klass_name) ⇒ Object
Loads ri data for
klass_name. -
#load_method(klass_name, method_name) ⇒ Object
Loads ri data for
method_nameinklass_name. -
#load_page(page_name) ⇒ Object
Loads ri data for
page_name. -
#main ⇒ Object
Gets the main page for this RDoc store.
-
#main=(page) ⇒ Object
Sets the main page for this RDoc store.
-
#make_variable_map(variables) ⇒ Object
Converts the variable => ClassModule map
variablesfrom a C parser into a variable => class name map. -
#merge_rbs_signatures(signatures) ⇒ Object
Stores RBS type signatures loaded from sidecar .rbs files, keyed by “ClassName#method” or “ClassName.method”.
-
#method_file(klass_name, method_name) ⇒ Object
Path to the ri data for
method_nameinklass_name. -
#module_names ⇒ Object
Modules cache accessor.
-
#modules_hash ⇒ Object
Hash of all modules known to RDoc.
-
#page(name) ⇒ Object
Returns the RDoc::TopLevel that is a file and has the given
name. -
#page_file(page_name) ⇒ Object
Path to the ri data for
page_name. -
#rbs_signature_for(method_attr) ⇒ Object
Returns the RBS type signature lines for
method_attrfrom loaded sidecar.rbsfiles, ornilif none. -
#remove_file(relative_name) ⇒ Object
Removes a file and its classes/modules from the store.
-
#remove_nodoc(all_hash) ⇒ Object
Removes from
all_hashthe contexts that are nodoc or have no content. -
#resolve_c_superclasses ⇒ Object
Make sure any references to C variable names are resolved to the corresponding class.
-
#save ⇒ Object
Saves all entries in the store.
-
#save_cache ⇒ Object
Writes the cache file for this store.
-
#save_class(klass) ⇒ Object
Writes the ri data for
klass(or module). -
#save_method(klass, method) ⇒ Object
Writes the ri data for
methodonklass. -
#save_page(page) ⇒ Object
Writes the ri data for
page. -
#source ⇒ Object
Source of the contents of this store.
-
#title ⇒ Object
Gets the title for this RDoc store.
-
#title=(title) ⇒ Object
Sets the title page for this RDoc store.
- #type_name_lookup ⇒ Object
-
#unique_classes ⇒ Object
Returns the unique classes discovered by RDoc.
-
#unique_classes_and_modules ⇒ Object
Returns the unique classes and modules discovered by RDoc.
-
#unique_modules ⇒ Object
Returns the unique modules discovered by RDoc.
Constructor Details
#initialize(options, path: nil, type: nil) ⇒ Store
Creates a new Store of type that will load or save to path
123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 |
# File 'lib/rdoc/store.rb', line 123 def initialize(, path: nil, type: nil) @options = @dry_run = .dry_run @encoding = .encoding @path = path || .op_dir @type = type @cache = { :ancestors => {}, :attributes => {}, :class_methods => {}, :c_class_variables => {}, :c_singleton_class_variables => {}, :encoding => @encoding, :instance_methods => {}, :main => .main_page, :modules => [], :pages => [], :title => .title, } @classes_hash = {} @modules_hash = {} @files_hash = {} @text_files_hash = {} @c_enclosure_classes = {} @c_enclosure_names = {} @c_class_variables = {} @c_singleton_class_variables = {} @unique_classes = nil @unique_modules = nil @unmatched_constant_alias = {} end |
Instance Attribute Details
#c_class_variables ⇒ Object (readonly)
Maps C variables to class or module names for each parsed C file.
80 81 82 |
# File 'lib/rdoc/store.rb', line 80 def c_class_variables @c_class_variables end |
#c_enclosure_classes ⇒ Object (readonly)
Stores the name of the C variable a class belongs to. This helps wire up classes defined from C across files.
73 74 75 |
# File 'lib/rdoc/store.rb', line 73 def c_enclosure_classes @c_enclosure_classes end |
#c_enclosure_names ⇒ Object (readonly)
:nodoc:
75 76 77 |
# File 'lib/rdoc/store.rb', line 75 def c_enclosure_names @c_enclosure_names end |
#c_singleton_class_variables ⇒ Object (readonly)
Maps C variables to singleton class names for each parsed C file.
85 86 87 |
# File 'lib/rdoc/store.rb', line 85 def c_singleton_class_variables @c_singleton_class_variables end |
#cache ⇒ Object (readonly)
The contents of the Store
108 109 110 |
# File 'lib/rdoc/store.rb', line 108 def cache @cache end |
#dry_run ⇒ Object
If true this Store will not write any files
90 91 92 |
# File 'lib/rdoc/store.rb', line 90 def dry_run @dry_run end |
#encoding ⇒ Object
The encoding of the contents in the Store
113 114 115 |
# File 'lib/rdoc/store.rb', line 113 def encoding @encoding end |
#options ⇒ Object (readonly)
Returns the value of attribute options.
97 98 99 |
# File 'lib/rdoc/store.rb', line 97 def @options end |
#path ⇒ Object
Path this store reads or writes
95 96 97 |
# File 'lib/rdoc/store.rb', line 95 def path @path end |
#type ⇒ Object
Type of ri datastore this was loaded from. See RDoc::RI::Driver, RDoc::RI::Paths.
103 104 105 |
# File 'lib/rdoc/store.rb', line 103 def type @type end |
#unmatched_constant_alias ⇒ Object (readonly)
The lazy constants alias will be discovered in passing
118 119 120 |
# File 'lib/rdoc/store.rb', line 118 def unmatched_constant_alias @unmatched_constant_alias end |
Instance Method Details
#add_c_enclosure(variable, namespace) ⇒ Object
Adds module as an enclosure (namespace) for the given variable for C files.
165 166 167 |
# File 'lib/rdoc/store.rb', line 165 def add_c_enclosure(variable, namespace) @c_enclosure_classes[variable] = namespace end |
#add_c_variables(c_parser) ⇒ Object
Adds C variables from an RDoc::Parser::C
172 173 174 175 176 177 178 |
# File 'lib/rdoc/store.rb', line 172 def add_c_variables(c_parser) filename = c_parser.top_level.relative_name @c_class_variables[filename] = make_variable_map c_parser.classes @c_singleton_class_variables[filename] = c_parser.singleton_classes end |
#add_file(absolute_name, relative_name: absolute_name, parser: nil) ⇒ Object
Adds the file with name as an RDoc::TopLevel to the store. Returns the created RDoc::TopLevel.
184 185 186 187 188 189 190 191 192 193 194 |
# File 'lib/rdoc/store.rb', line 184 def add_file(absolute_name, relative_name: absolute_name, parser: nil) unless top_level = @files_hash[relative_name] then top_level = RDoc::TopLevel.new absolute_name, relative_name top_level.parser = parser if parser top_level.store = self @files_hash[relative_name] = top_level @text_files_hash[relative_name] = top_level if top_level.text? end top_level end |
#all_classes ⇒ Object
Returns all classes discovered by RDoc
335 336 337 |
# File 'lib/rdoc/store.rb', line 335 def all_classes @classes_hash.values end |
#all_classes_and_modules ⇒ Object
Returns all classes and modules discovered by RDoc
342 343 344 |
# File 'lib/rdoc/store.rb', line 342 def all_classes_and_modules @classes_hash.values + @modules_hash.values end |
#all_files ⇒ Object
All TopLevels known to RDoc
429 430 431 |
# File 'lib/rdoc/store.rb', line 429 def all_files @files_hash.values end |
#all_modules ⇒ Object
Returns all modules discovered by RDoc
436 437 438 |
# File 'lib/rdoc/store.rb', line 436 def all_modules modules_hash.values end |
#ancestors ⇒ Object
Ancestors cache accessor. Maps a klass name to an Array of its ancestors in this store. If Foo in this store inherits from Object, Kernel won’t be listed (it will be included from ruby’s ri store).
445 446 447 |
# File 'lib/rdoc/store.rb', line 445 def ancestors @cache[:ancestors] end |
#attributes ⇒ Object
Attributes cache accessor. Maps a class to an Array of its attributes.
452 453 454 |
# File 'lib/rdoc/store.rb', line 452 def attributes @cache[:attributes] end |
#cache_path ⇒ Object
Path to the cache file
459 460 461 |
# File 'lib/rdoc/store.rb', line 459 def cache_path File.join @path, 'cache.ri' end |
#cache_text_file(relative_name) ⇒ Object
Caches relative_name in the text files hash, if it is a text file.
326 327 328 329 330 |
# File 'lib/rdoc/store.rb', line 326 def cache_text_file(relative_name) if top_level = @files_hash[relative_name] @text_files_hash[relative_name] = top_level if top_level.text? end end |
#class_file(klass_name) ⇒ Object
Path to the ri data for klass_name
466 467 468 469 |
# File 'lib/rdoc/store.rb', line 466 def class_file(klass_name) name = klass_name.split('::').last File.join class_path(klass_name), "cdesc-#{name}.ri" end |
#class_methods ⇒ Object
Class methods cache accessor. Maps a class to an Array of its class methods (not full name).
475 476 477 |
# File 'lib/rdoc/store.rb', line 475 def class_methods @cache[:class_methods] end |
#class_path(klass_name) ⇒ Object
Path where data for klass_name will be stored (methods or class data)
482 483 484 |
# File 'lib/rdoc/store.rb', line 482 def class_path(klass_name) File.join @path, *klass_name.split('::') end |
#classes_hash ⇒ Object
Hash of all classes known to RDoc
489 490 491 |
# File 'lib/rdoc/store.rb', line 489 def classes_hash @classes_hash end |
#clean_cache_collection(collection) ⇒ Object
Removes empty items and ensures item in each collection are unique and sorted
497 498 499 500 501 502 503 504 505 506 507 |
# File 'lib/rdoc/store.rb', line 497 def clean_cache_collection(collection) # :nodoc: collection.each do |name, item| if item.empty? then collection.delete name else # HACK mongrel-1.1.5 documents its files twice item.uniq! item.sort! end end end |
#cleanup_stale_contributions ⇒ Object
Removes stale empty placeholders left by clear_file_contributions with keep_position: true. After re-parsing, a file may no longer define a class it previously contributed to, leaving an empty entry in comment_location and a stale in_files reference. Call this after all re-parsing is complete.
289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'lib/rdoc/store.rb', line 289 def cleanup_stale_contributions all_classes_and_modules.each do |cm| cm.comment_location.delete_if { |_, comments| comments.empty? } cm.rebuild_comment_from_location cm.in_files.select! { |tl| cm.comment_location.key?(tl) || cm.method_list.any? { |m| m.file == tl } || cm.attributes.any? { |a| a.file == tl } || cm.constants.any? { |c| c.file == tl } } if cm.in_files.empty? if cm.is_a?(RDoc::NormalModule) @modules_hash.delete(cm.full_name) else @classes_hash.delete(cm.full_name) end cm.parent&.classes_hash&.delete(cm.name) cm.parent&.modules_hash&.delete(cm.name) end end end |
#clear_file_contributions(relative_name, keep_position: false) ⇒ Object
Removes a file’s contributions (methods, constants, comments, etc.) from its classes and modules. If no other files contribute to a class or module, it is removed from the store entirely. This prevents duplication when the file is re-parsed while preserving shared namespaces like RDoc that span many files.
223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 |
# File 'lib/rdoc/store.rb', line 223 def clear_file_contributions(relative_name, keep_position: false) top_level = @files_hash[relative_name] return unless top_level top_level.classes_or_modules.each do |cm| # Remove methods and attributes contributed by this file cm.method_list.reject! { |m| m.file == top_level } cm.attributes.reject! { |a| a.file == top_level } # Rebuild methods_hash from remaining methods and attributes cm.methods_hash.clear cm.method_list.each { |m| cm.methods_hash[m.pretty_name] = m } cm.attributes.each { |a| cm.methods_hash[a.pretty_name] = a } # Remove constants contributed by this file cm.constants.reject! { |c| c.file == top_level } cm.constants_hash.clear cm.constants.each { |c| cm.constants_hash[c.name] = c } # Remove includes, extends, and aliases from this file cm.includes.reject! { |i| i.file == top_level } cm.extends.reject! { |e| e.file == top_level } cm.aliases.reject! { |a| a.file == top_level } cm.external_aliases.reject! { |a| a.file == top_level } # Clear or remove comment entries from this file if cm.is_a?(RDoc::ClassModule) if keep_position cm.comment_location[top_level] = [] if cm.comment_location.key?(top_level) else cm.comment_location.delete(top_level) end cm.rebuild_comment_from_location end unless keep_position # Remove this file from the class/module's file list cm.in_files.delete(top_level) # If no files contribute to this class/module anymore, remove it # from the store entirely. This handles file deletion correctly # for classes that are only defined in the deleted file, while # preserving classes that span multiple files. if cm.in_files.empty? if cm.is_a?(RDoc::NormalModule) @modules_hash.delete(cm.full_name) else @classes_hash.delete(cm.full_name) end cm.parent&.classes_hash&.delete(cm.name) cm.parent&.modules_hash&.delete(cm.name) end end end # Clear the TopLevel's class/module list to prevent duplicates top_level.classes_or_modules.clear end |
#clear_rbs_signatures ⇒ Object
:nodoc:
422 423 424 |
# File 'lib/rdoc/store.rb', line 422 def clear_rbs_signatures # :nodoc: @rbs_signatures = nil end |
#complete(min_visibility) ⇒ Object
Prepares the RDoc code object tree for use by a generator.
It finds unique classes/modules defined, and replaces classes/modules that are aliases for another one by a copy with RDoc::ClassModule#is_alias_for set.
It updates the RDoc::ClassModule#constant_aliases attribute of “real” classes or modules.
It also completely removes the classes and modules that should be removed from the documentation and the methods that have a visibility below min_visibility, which is the --visibility option.
See also RDoc::Context#remove_from_documentation?
525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 |
# File 'lib/rdoc/store.rb', line 525 def complete(min_visibility) fix_basic_object_inheritance # cache included modules before they are removed from the documentation all_classes_and_modules.each { |cm| cm.ancestors } unless min_visibility == :nodoc then remove_nodoc @classes_hash remove_nodoc @modules_hash end @unique_classes = find_unique @classes_hash @unique_modules = find_unique @modules_hash unique_classes_and_modules.each do |cm| cm.complete min_visibility end @files_hash.each_key do |file_name| tl = @files_hash[file_name] unless tl.text? then tl.modules_hash.clear tl.classes_hash.clear tl.classes_or_modules.each do |cm| name = cm.full_name if cm.type == 'class' then tl.classes_hash[name] = cm if @classes_hash[name] else tl.modules_hash[name] = cm if @modules_hash[name] end end end end end |
#files_hash ⇒ Object
Hash of all files known to RDoc
565 566 567 |
# File 'lib/rdoc/store.rb', line 565 def files_hash @files_hash end |
#find_c_enclosure(variable) ⇒ Object
Finds the enclosure (namespace) for the given C variable.
572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 |
# File 'lib/rdoc/store.rb', line 572 def find_c_enclosure(variable) @c_enclosure_classes.fetch variable do break unless name = @c_enclosure_names[variable] mod = find_class_or_module name unless mod then loaded_mod = load_class_data name file = loaded_mod.in_files.first return unless file # legacy data source file.store = self mod = file.add_module RDoc::NormalModule, name end @c_enclosure_classes[variable] = mod end end |
#find_class_named(name) ⇒ Object
Finds the class with name in all discovered classes
597 598 599 |
# File 'lib/rdoc/store.rb', line 597 def find_class_named(name) @classes_hash[name] end |
#find_class_named_from(name, from) ⇒ Object
Finds the class with name starting in namespace from
604 605 606 607 608 609 610 611 612 613 614 615 616 617 |
# File 'lib/rdoc/store.rb', line 604 def find_class_named_from(name, from) from = find_class_named from unless RDoc::Context === from until RDoc::TopLevel === from do return nil unless from klass = from.find_class_named name return klass if klass from = from.parent end find_class_named name end |
#find_class_or_module(name) ⇒ Object
Finds the class or module with name
622 623 624 625 |
# File 'lib/rdoc/store.rb', line 622 def find_class_or_module(name) name = $' if name =~ /^::/ @classes_hash[name] || @modules_hash[name] end |
#find_file_named(name) ⇒ Object
Finds the file with name in all discovered files
630 631 632 |
# File 'lib/rdoc/store.rb', line 630 def find_file_named(name) @files_hash[name] end |
#find_module_named(name) ⇒ Object
Finds the module with name in all discovered modules
637 638 639 |
# File 'lib/rdoc/store.rb', line 637 def find_module_named(name) @modules_hash[name] end |
#find_text_page(file_name) ⇒ Object
Returns the RDoc::TopLevel that is a text file and has the given file_name
645 646 647 648 649 |
# File 'lib/rdoc/store.rb', line 645 def find_text_page(file_name) @text_files_hash.each_value.find do |file| file.full_name == file_name end end |
#find_unique(all_hash) ⇒ Object
Finds unique classes/modules defined in all_hash, and returns them as an array. Performs the alias updates in all_hash: see ::complete. – TODO aliases should be registered by Context#add_module_alias
658 659 660 661 662 663 664 665 666 |
# File 'lib/rdoc/store.rb', line 658 def find_unique(all_hash) unique = [] all_hash.each_pair do |full_name, cm| unique << cm if full_name == cm.full_name end unique end |
#fix_basic_object_inheritance ⇒ Object
Fixes the erroneous BasicObject < Object in 1.9.
Because we assumed all classes without a stated superclass inherit from Object, we have the above wrong inheritance.
We fix BasicObject right away if we are running in a Ruby version >= 1.9.
677 678 679 680 681 |
# File 'lib/rdoc/store.rb', line 677 def fix_basic_object_inheritance basic = classes_hash['BasicObject'] return unless basic basic.superclass = nil end |
#friendly_path ⇒ Object
Friendly rendition of #path
686 687 688 689 690 691 692 693 694 695 696 |
# File 'lib/rdoc/store.rb', line 686 def friendly_path case type when :gem then parent = File. '..', @path "gem #{File.basename parent}" when :home then RDoc.home when :site then 'ruby site' when :system then 'ruby core' else @path end end |
#inspect ⇒ Object
:nodoc:
698 699 700 |
# File 'lib/rdoc/store.rb', line 698 def inspect # :nodoc: "#<%s:0x%x %s %p>" % [self.class, object_id, @path, module_names.sort] end |
#instance_methods ⇒ Object
Instance methods cache accessor. Maps a class to an Array of its instance methods (not full name).
706 707 708 |
# File 'lib/rdoc/store.rb', line 706 def instance_methods @cache[:instance_methods] end |
#invalidate_type_name_lookup ⇒ Object
Invalidates the cached type name lookup. Server mode calls this after re-parsing changes the set of classes and modules.
357 358 359 |
# File 'lib/rdoc/store.rb', line 357 def invalidate_type_name_lookup # :nodoc: @type_name_lookup = nil end |
#load_all ⇒ Object
Loads all items from this store into memory. This recreates a documentation tree for use by a generator
714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 |
# File 'lib/rdoc/store.rb', line 714 def load_all load_cache module_names.each do |module_name| mod = find_class_or_module(module_name) || load_class(module_name) # load method documentation since the loaded class/module does not have # it loaded_methods = mod.method_list.map do |method| load_method module_name, method.full_name end mod.method_list.replace loaded_methods loaded_attributes = mod.attributes.map do |attribute| load_method module_name, attribute.full_name end mod.attributes.replace loaded_attributes end all_classes_and_modules.each do |mod| descendent_re = /^#{mod.full_name}::[^:]+$/ module_names.each do |name| next unless name =~ descendent_re descendent = find_class_or_module name case descendent when RDoc::NormalClass then mod.classes_hash[name] = descendent when RDoc::NormalModule then mod.modules_hash[name] = descendent end end end @cache[:pages].each do |page_name| page = load_page page_name @files_hash[page_name] = page @text_files_hash[page_name] = page if page.text? end end |
#load_cache ⇒ Object
Loads cache file for this store
762 763 764 765 766 767 768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 784 785 786 787 788 789 790 791 792 793 794 795 796 |
# File 'lib/rdoc/store.rb', line 762 def load_cache #orig_enc = @encoding @cache = marshal_load(cache_path) load_enc = @cache[:encoding] # TODO this feature will be time-consuming to add: # a) Encodings may be incompatible but transcodeable # b) Need to warn in the appropriate spots, wherever they may be # c) Need to handle cross-cache differences in encodings # d) Need to warn when generating into a cache with different encodings # #if orig_enc and load_enc != orig_enc then # warn "Cached encoding #{load_enc} is incompatible with #{orig_enc}\n" \ # "from #{path}/cache.ri" unless # Encoding.compatible? orig_enc, load_enc #end @encoding = load_enc unless @encoding @cache[:pages] ||= [] @cache[:main] ||= nil @cache[:c_class_variables] ||= {} @cache[:c_singleton_class_variables] ||= {} @cache[:c_class_variables].each do |_, map| map.each do |variable, name| @c_enclosure_names[variable] = name end end @cache rescue Errno::ENOENT end |
#load_class(klass_name) ⇒ Object
Loads ri data for klass_name and hooks it up to this store.
801 802 803 804 805 806 807 808 809 810 811 812 813 814 |
# File 'lib/rdoc/store.rb', line 801 def load_class(klass_name) obj = load_class_data klass_name obj.store = self case obj when RDoc::NormalClass then @classes_hash[klass_name] = obj when RDoc::SingleClass then @classes_hash[klass_name] = obj when RDoc::NormalModule then @modules_hash[klass_name] = obj end end |
#load_class_data(klass_name) ⇒ Object
Loads ri data for klass_name
819 820 821 822 823 824 825 826 827 |
# File 'lib/rdoc/store.rb', line 819 def load_class_data(klass_name) file = class_file klass_name marshal_load(file) rescue Errno::ENOENT => e error = MissingFileError.new(self, file, klass_name) error.set_backtrace e.backtrace raise error end |
#load_method(klass_name, method_name) ⇒ Object
Loads ri data for method_name in klass_name
832 833 834 835 836 837 838 839 840 841 842 843 |
# File 'lib/rdoc/store.rb', line 832 def load_method(klass_name, method_name) file = method_file klass_name, method_name obj = marshal_load(file) obj.store = self obj.parent ||= find_class_or_module(klass_name) || load_class(klass_name) obj rescue Errno::ENOENT => e error = MissingFileError.new(self, file, klass_name + method_name) error.set_backtrace e.backtrace raise error end |
#load_page(page_name) ⇒ Object
Loads ri data for page_name
848 849 850 851 852 853 854 855 856 857 858 |
# File 'lib/rdoc/store.rb', line 848 def load_page(page_name) file = page_file page_name obj = marshal_load(file) obj.store = self obj rescue Errno::ENOENT => e error = MissingFileError.new(self, file, page_name) error.set_backtrace e.backtrace raise error end |
#main ⇒ Object
Gets the main page for this RDoc store. This page is used as the root of the RDoc server.
864 865 866 |
# File 'lib/rdoc/store.rb', line 864 def main @cache[:main] end |
#main=(page) ⇒ Object
Sets the main page for this RDoc store.
871 872 873 |
# File 'lib/rdoc/store.rb', line 871 def main=(page) @cache[:main] = page end |
#make_variable_map(variables) ⇒ Object
Converts the variable => ClassModule map variables from a C parser into a variable => class name map.
879 880 881 882 883 884 885 886 887 |
# File 'lib/rdoc/store.rb', line 879 def make_variable_map(variables) map = {} variables.each { |variable, class_module| map[variable] = class_module.full_name } map end |
#merge_rbs_signatures(signatures) ⇒ Object
Stores RBS type signatures loaded from sidecar .rbs files, keyed by “ClassName#method” or “ClassName.method”. Replaces any previously stored set, so passing {} clears it. Inline #: annotations on method objects are NOT touched — those are owned by the source file.
389 390 391 |
# File 'lib/rdoc/store.rb', line 389 def merge_rbs_signatures(signatures) @rbs_signatures = signatures end |
#method_file(klass_name, method_name) ⇒ Object
Path to the ri data for method_name in klass_name
892 893 894 895 896 897 898 899 900 |
# File 'lib/rdoc/store.rb', line 892 def method_file(klass_name, method_name) method_name = method_name.split('::').last method_name =~ /#(.*)/ method_type = $1 ? 'i' : 'c' method_name = $1 if $1 method_name = method_name.gsub(/\W/) { "%%%02x" % $&[0].ord } File.join class_path(klass_name), "#{method_name}-#{method_type}.ri" end |
#module_names ⇒ Object
Modules cache accessor. An Array of all the module (and class) names in the store.
906 907 908 |
# File 'lib/rdoc/store.rb', line 906 def module_names @cache[:modules] end |
#modules_hash ⇒ Object
Hash of all modules known to RDoc
913 914 915 |
# File 'lib/rdoc/store.rb', line 913 def modules_hash @modules_hash end |
#page(name) ⇒ Object
Returns the RDoc::TopLevel that is a file and has the given name
920 921 922 923 924 |
# File 'lib/rdoc/store.rb', line 920 def page(name) @files_hash.each_value.find do |file| file.page_name == name or file.base_name == name end end |
#page_file(page_name) ⇒ Object
Path to the ri data for page_name
929 930 931 932 933 |
# File 'lib/rdoc/store.rb', line 929 def page_file(page_name) file_name = File.basename(page_name).gsub('.', '_') File.join @path, File.dirname(page_name), "page-#{file_name}.ri" end |
#rbs_signature_for(method_attr) ⇒ Object
Returns the RBS type signature lines for method_attr from loaded sidecar .rbs files, or nil if none. Falls through to the canonical method for aliases, and handles initialize -> .new singleton mapping.
399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 |
# File 'lib/rdoc/store.rb', line 399 def rbs_signature_for(method_attr) return nil unless @rbs_signatures cm = method_attr.parent return nil unless cm.respond_to?(:full_name) key = method_attr.singleton ? "#{cm.full_name}.#{method_attr.name}" : "#{cm.full_name}##{method_attr.name}" sig = @rbs_signatures[key] # RBS keys constructors as #initialize, but RDoc renames them to .new if !sig && method_attr.name == 'new' && method_attr.singleton sig = @rbs_signatures["#{cm.full_name}#initialize"] end # For aliases, fall through to the canonical method (its inline #: # takes precedence over any sidecar signature on the alias's name). if !sig && method_attr.is_alias_for canonical = method_attr.is_alias_for sig = canonical.type_signature_lines || rbs_signature_for(canonical) end sig end |
#remove_file(relative_name) ⇒ Object
Removes a file and its classes/modules from the store. Used by the live-reloading server when a source file is deleted.
Note: this does not handle reopened classes correctly. If a class is defined across multiple files (e.g. Foo in both a.rb and b.rb), deleting one file removes the entire class from the store — including the parts contributed by the other file. Saving the remaining file triggers a re-parse that restores it.
206 207 208 209 210 211 212 213 214 |
# File 'lib/rdoc/store.rb', line 206 def remove_file(relative_name) top_level = @files_hash.delete(relative_name) @text_files_hash.delete(relative_name) @c_class_variables.delete(relative_name) @c_singleton_class_variables.delete(relative_name) return unless top_level remove_classes_and_modules(top_level.classes_or_modules) end |
#remove_nodoc(all_hash) ⇒ Object
Removes from all_hash the contexts that are nodoc or have no content.
See RDoc::Context#remove_from_documentation?
940 941 942 943 944 945 |
# File 'lib/rdoc/store.rb', line 940 def remove_nodoc(all_hash) all_hash.keys.each do |name| context = all_hash[name] all_hash.delete(name) if context.remove_from_documentation? end end |
#resolve_c_superclasses ⇒ Object
Make sure any references to C variable names are resolved to the corresponding class.
315 316 317 318 319 320 321 |
# File 'lib/rdoc/store.rb', line 315 def resolve_c_superclasses @classes_hash.each_value do |klass| if klass.superclass.is_a?(String) && (candidate = find_c_enclosure(klass.superclass)) klass.superclass = candidate end end end |
#save ⇒ Object
Saves all entries in the store
950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 |
# File 'lib/rdoc/store.rb', line 950 def save load_cache all_classes_and_modules.each do |klass| save_class klass klass.each_method do |method| save_method klass, method end klass.attributes.each do |attribute| save_method klass, attribute end end all_files.each do |file| save_page file end save_cache end |
#save_cache ⇒ Object
Writes the cache file for this store
975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 |
# File 'lib/rdoc/store.rb', line 975 def save_cache clean_cache_collection @cache[:ancestors] clean_cache_collection @cache[:attributes] clean_cache_collection @cache[:class_methods] clean_cache_collection @cache[:instance_methods] @cache[:modules].uniq! @cache[:modules].sort! @cache[:pages].uniq! @cache[:pages].sort! @cache[:encoding] = @encoding # this gets set twice due to assert_cache @cache[:c_class_variables].merge! @c_class_variables @cache[:c_singleton_class_variables].merge! @c_singleton_class_variables return if @dry_run File.open cache_path, 'wb' do |io| Marshal.dump @cache, io end end |
#save_class(klass) ⇒ Object
Writes the ri data for klass (or module)
1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 1065 1066 1067 1068 1069 1070 1071 |
# File 'lib/rdoc/store.rb', line 1002 def save_class(klass) full_name = klass.full_name FileUtils.mkdir_p class_path(full_name) unless @dry_run @cache[:modules] << full_name path = class_file full_name begin disk_klass = load_class full_name klass = disk_klass.merge klass rescue MissingFileError end # BasicObject has no ancestors ancestors = klass.direct_ancestors.compact.map do |ancestor| # HACK for classes we don't know about (class X < RuntimeError) String === ancestor ? ancestor : ancestor.full_name end @cache[:ancestors][full_name] ||= [] @cache[:ancestors][full_name].concat ancestors attribute_definitions = klass.attributes.map do |attribute| "#{attribute.definition} #{attribute.name}" end unless attribute_definitions.empty? then @cache[:attributes][full_name] ||= [] @cache[:attributes][full_name].concat attribute_definitions end to_delete = [] unless klass.method_list.empty? then @cache[:class_methods][full_name] ||= [] @cache[:instance_methods][full_name] ||= [] class_methods, instance_methods = klass.method_list.partition { |meth| meth.singleton } class_methods = class_methods. map { |method| method.name } instance_methods = instance_methods.map { |method| method.name } attribute_names = klass.attributes.map { |attr| attr.name } old = @cache[:class_methods][full_name] - class_methods to_delete.concat old.map { |method| method_file full_name, "#{full_name}::#{method}" } old = @cache[:instance_methods][full_name] - instance_methods - attribute_names to_delete.concat old.map { |method| method_file full_name, "#{full_name}##{method}" } @cache[:class_methods][full_name] = class_methods @cache[:instance_methods][full_name] = instance_methods end return if @dry_run FileUtils.rm_f to_delete File.open path, 'wb' do |io| Marshal.dump klass, io end end |
#save_method(klass, method) ⇒ Object
Writes the ri data for method on klass
1076 1077 1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 |
# File 'lib/rdoc/store.rb', line 1076 def save_method(klass, method) full_name = klass.full_name FileUtils.mkdir_p class_path(full_name) unless @dry_run cache = if method.singleton then @cache[:class_methods] else @cache[:instance_methods] end cache[full_name] ||= [] cache[full_name] << method.name return if @dry_run File.open method_file(full_name, method.full_name), 'wb' do |io| Marshal.dump method, io end end |
#save_page(page) ⇒ Object
Writes the ri data for page
1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 |
# File 'lib/rdoc/store.rb', line 1099 def save_page(page) return unless page.text? path = page_file page.full_name FileUtils.mkdir_p File.dirname(path) unless @dry_run cache[:pages] ||= [] cache[:pages] << page.full_name return if @dry_run File.open path, 'wb' do |io| Marshal.dump page, io end end |
#source ⇒ Object
Source of the contents of this store.
For a store from a gem the source is the gem name. For a store from the home directory the source is “home”. For system ri store (the standard library documentation) the source is“ruby”. For a store from the site ri directory the store is “site”. For other stores the source is the #path.
1125 1126 1127 1128 1129 1130 1131 1132 1133 |
# File 'lib/rdoc/store.rb', line 1125 def source case type when :gem then File.basename File. '..', @path when :home then 'home' when :site then 'site' when :system then 'ruby' else @path end end |
#title ⇒ Object
Gets the title for this RDoc store. This is used as the title in each page on the RDoc server
1139 1140 1141 |
# File 'lib/rdoc/store.rb', line 1139 def title @cache[:title] end |
#title=(title) ⇒ Object
Sets the title page for this RDoc store.
1146 1147 1148 |
# File 'lib/rdoc/store.rb', line 1146 def title=(title) @cache[:title] = title end |
#type_name_lookup ⇒ Object
361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 |
# File 'lib/rdoc/store.rb', line 361 def type_name_lookup @type_name_lookup ||= begin lookup = {} unqualified_names = {} ambiguous_names = {} all_classes_and_modules.each do |cm| lookup[cm.full_name] = cm.path unqualified_name = cm.name if ambiguous_names[unqualified_name] # already known ambiguous, skip elsif unqualified_names.key?(unqualified_name) unqualified_names.delete(unqualified_name) ambiguous_names[unqualified_name] = true else unqualified_names[unqualified_name] = cm.path end end lookup.merge!(unqualified_names) end end |
#unique_classes ⇒ Object
Returns the unique classes discovered by RDoc.
::complete must have been called prior to using this method.
1155 1156 1157 |
# File 'lib/rdoc/store.rb', line 1155 def unique_classes @unique_classes end |
#unique_classes_and_modules ⇒ Object
Returns the unique classes and modules discovered by RDoc. ::complete must have been called prior to using this method.
1163 1164 1165 |
# File 'lib/rdoc/store.rb', line 1163 def unique_classes_and_modules @unique_classes + @unique_modules end |
#unique_modules ⇒ Object
Returns the unique modules discovered by RDoc. ::complete must have been called prior to using this method.
1171 1172 1173 |
# File 'lib/rdoc/store.rb', line 1171 def unique_modules @unique_modules end |